# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/SettingsWidget.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SettingsWidget(object):
    def setupUi(self, SettingsWidget):
        SettingsWidget.setObjectName("SettingsWidget")
        SettingsWidget.resize(417, 697)
        SettingsWidget.setMinimumSize(QtCore.QSize(0, 0))
        SettingsWidget.setMaximumSize(QtCore.QSize(457, 16777215))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(SettingsWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(SettingsWidget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.frame_source = QtWidgets.QFrame(self.groupBox)
        self.frame_source.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_source.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_source.setObjectName("frame_source")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.frame_source)
        self.horizontalLayout_5.setContentsMargins(3, 3, 3, 3)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_26 = QtWidgets.QLabel(self.frame_source)
        self.label_26.setMinimumSize(QtCore.QSize(125, 0))
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_5.addWidget(self.label_26)
        self.cmb_source = QtWidgets.QComboBox(self.frame_source)
        self.cmb_source.setMinimumSize(QtCore.QSize(200, 0))
        self.cmb_source.setObjectName("cmb_source")
        self.horizontalLayout_5.addWidget(self.cmb_source)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.verticalLayout_3.addWidget(self.frame_source)
        self.frame_folder = QtWidgets.QFrame(self.groupBox)
        self.frame_folder.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_folder.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_folder.setObjectName("frame_folder")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.frame_folder)
        self.horizontalLayout_4.setContentsMargins(3, 3, 3, 3)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_23 = QtWidgets.QLabel(self.frame_folder)
        self.label_23.setMinimumSize(QtCore.QSize(125, 0))
        self.label_23.setObjectName("label_23")
        self.horizontalLayout_4.addWidget(self.label_23)
        self.cmb_path = QtWidgets.QComboBox(self.frame_folder)
        self.cmb_path.setMinimumSize(QtCore.QSize(200, 0))
        self.cmb_path.setObjectName("cmb_path")
        self.horizontalLayout_4.addWidget(self.cmb_path)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.verticalLayout_3.addWidget(self.frame_folder)
        self.frame_exposure = QtWidgets.QFrame(self.groupBox)
        self.frame_exposure.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_exposure.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_exposure.setObjectName("frame_exposure")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.frame_exposure)
        self.horizontalLayout_6.setContentsMargins(3, 3, 3, 3)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label = QtWidgets.QLabel(self.frame_exposure)
        self.label.setMinimumSize(QtCore.QSize(125, 0))
        self.label.setObjectName("label")
        self.horizontalLayout_6.addWidget(self.label)
        self.sb_exposure = QtWidgets.QDoubleSpinBox(self.frame_exposure)
        self.sb_exposure.setMinimumSize(QtCore.QSize(75, 0))
        self.sb_exposure.setMaximumSize(QtCore.QSize(75, 16777215))
        self.sb_exposure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sb_exposure.setDecimals(6)
        self.sb_exposure.setMinimum(1e-06)
        self.sb_exposure.setProperty("value", 1.0)
        self.sb_exposure.setObjectName("sb_exposure")
        self.horizontalLayout_6.addWidget(self.sb_exposure)
        self.label_3 = QtWidgets.QLabel(self.frame_exposure)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_6.addWidget(self.label_3)
        self.sb_gain = QtWidgets.QSpinBox(self.frame_exposure)
        self.sb_gain.setMinimumSize(QtCore.QSize(75, 0))
        self.sb_gain.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sb_gain.setMaximum(22)
        self.sb_gain.setObjectName("sb_gain")
        self.horizontalLayout_6.addWidget(self.sb_gain)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.verticalLayout_3.addWidget(self.frame_exposure)
        self.verticalLayout_2.addWidget(self.groupBox)
        self.groupBox_4 = QtWidgets.QGroupBox(SettingsWidget)
        self.groupBox_4.setObjectName("groupBox_4")
        self.vl_levels = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.vl_levels.setObjectName("vl_levels")
        self.chk_auto_levels = QtWidgets.QCheckBox(self.groupBox_4)
        self.chk_auto_levels.setChecked(True)
        self.chk_auto_levels.setObjectName("chk_auto_levels")
        self.vl_levels.addWidget(self.chk_auto_levels)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_2 = QtWidgets.QLabel(self.groupBox_4)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_11.addWidget(self.label_2)
        self.sb_min_level = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_min_level.sizePolicy().hasHeightForWidth())
        self.sb_min_level.setSizePolicy(sizePolicy)
        self.sb_min_level.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sb_min_level.setMaximum(300.0)
        self.sb_min_level.setObjectName("sb_min_level")
        self.horizontalLayout_11.addWidget(self.sb_min_level)
        self.label_4 = QtWidgets.QLabel(self.groupBox_4)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_11.addWidget(self.label_4)
        self.sb_max_level = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_max_level.sizePolicy().hasHeightForWidth())
        self.sb_max_level.setSizePolicy(sizePolicy)
        self.sb_max_level.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sb_max_level.setMaximum(300.0)
        self.sb_max_level.setProperty("value", 1.0)
        self.sb_max_level.setObjectName("sb_max_level")
        self.horizontalLayout_11.addWidget(self.sb_max_level)
        self.vl_levels.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.rb_lin_level = QtWidgets.QRadioButton(self.groupBox_4)
        self.rb_lin_level.setChecked(True)
        self.rb_lin_level.setObjectName("rb_lin_level")
        self.bg_level = QtWidgets.QButtonGroup(SettingsWidget)
        self.bg_level.setObjectName("bg_level")
        self.bg_level.addButton(self.rb_lin_level)
        self.horizontalLayout_13.addWidget(self.rb_lin_level)
        self.rb_log_level = QtWidgets.QRadioButton(self.groupBox_4)
        self.rb_log_level.setObjectName("rb_log_level")
        self.bg_level.addButton(self.rb_log_level)
        self.horizontalLayout_13.addWidget(self.rb_log_level)
        self.rb_sqrt_level = QtWidgets.QRadioButton(self.groupBox_4)
        self.rb_sqrt_level.setObjectName("rb_sqrt_level")
        self.bg_level.addButton(self.rb_sqrt_level)
        self.horizontalLayout_13.addWidget(self.rb_sqrt_level)
        self.vl_levels.addLayout(self.horizontalLayout_13)
        self.verticalLayout_2.addWidget(self.groupBox_4)
        self.chk_additional_settings = QtWidgets.QCheckBox(SettingsWidget)
        self.chk_additional_settings.setObjectName("chk_additional_settings")
        self.verticalLayout_2.addWidget(self.chk_additional_settings)
        self.gb_ext_settings = QtWidgets.QGroupBox(SettingsWidget)
        self.gb_ext_settings.setCheckable(False)
        self.gb_ext_settings.setObjectName("gb_ext_settings")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.gb_ext_settings)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame_Cut = QtWidgets.QFrame(self.gb_ext_settings)
        self.frame_Cut.setObjectName("frame_Cut")
        self.gridLayout = QtWidgets.QGridLayout(self.frame_Cut)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 0, 5, 1, 1)
        self.sb_view_h = QtWidgets.QSpinBox(self.frame_Cut)
        self.sb_view_h.setMinimum(-1)
        self.sb_view_h.setMaximum(9999)
        self.sb_view_h.setSingleStep(4)
        self.sb_view_h.setObjectName("sb_view_h")
        self.gridLayout.addWidget(self.sb_view_h, 1, 4, 1, 1)
        self.label_22 = QtWidgets.QLabel(self.frame_Cut)
        self.label_22.setMinimumSize(QtCore.QSize(100, 0))
        self.label_22.setObjectName("label_22")
        self.gridLayout.addWidget(self.label_22, 0, 0, 2, 1)
        self.sb_view_w = QtWidgets.QSpinBox(self.frame_Cut)
        self.sb_view_w.setMinimum(-1)
        self.sb_view_w.setMaximum(9999)
        self.sb_view_w.setSingleStep(4)
        self.sb_view_w.setObjectName("sb_view_w")
        self.gridLayout.addWidget(self.sb_view_w, 1, 2, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.frame_Cut)
        self.label_19.setMinimumSize(QtCore.QSize(20, 0))
        self.label_19.setObjectName("label_19")
        self.gridLayout.addWidget(self.label_19, 0, 1, 1, 1)
        self.sb_view_y = QtWidgets.QSpinBox(self.frame_Cut)
        self.sb_view_y.setMinimumSize(QtCore.QSize(75, 0))
        self.sb_view_y.setMaximum(9999)
        self.sb_view_y.setSingleStep(4)
        self.sb_view_y.setObjectName("sb_view_y")
        self.gridLayout.addWidget(self.sb_view_y, 0, 4, 1, 1)
        self.label_28 = QtWidgets.QLabel(self.frame_Cut)
        self.label_28.setObjectName("label_28")
        self.gridLayout.addWidget(self.label_28, 1, 3, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.frame_Cut)
        self.label_20.setMinimumSize(QtCore.QSize(25, 0))
        self.label_20.setObjectName("label_20")
        self.gridLayout.addWidget(self.label_20, 0, 3, 1, 1)
        self.sb_view_x = QtWidgets.QSpinBox(self.frame_Cut)
        self.sb_view_x.setMinimumSize(QtCore.QSize(75, 0))
        self.sb_view_x.setMaximum(9999)
        self.sb_view_x.setSingleStep(4)
        self.sb_view_x.setObjectName("sb_view_x")
        self.gridLayout.addWidget(self.sb_view_x, 0, 2, 1, 1)
        self.label_27 = QtWidgets.QLabel(self.frame_Cut)
        self.label_27.setObjectName("label_27")
        self.gridLayout.addWidget(self.label_27, 1, 1, 1, 1)
        self.verticalLayout.addWidget(self.frame_Cut)
        self.frame_reduce = QtWidgets.QFrame(self.gb_ext_settings)
        self.frame_reduce.setObjectName("frame_reduce")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.frame_reduce)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.lbFps_2 = QtWidgets.QLabel(self.frame_reduce)
        self.lbFps_2.setMinimumSize(QtCore.QSize(125, 0))
        self.lbFps_2.setObjectName("lbFps_2")
        self.horizontalLayout_9.addWidget(self.lbFps_2)
        self.sb_reduce = QtWidgets.QSpinBox(self.frame_reduce)
        self.sb_reduce.setMinimumSize(QtCore.QSize(75, 0))
        self.sb_reduce.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sb_reduce.setMinimum(1)
        self.sb_reduce.setObjectName("sb_reduce")
        self.horizontalLayout_9.addWidget(self.sb_reduce)
        self.lbFps_3 = QtWidgets.QLabel(self.frame_reduce)
        self.lbFps_3.setObjectName("lbFps_3")
        self.horizontalLayout_9.addWidget(self.lbFps_3)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem4)
        self.verticalLayout.addWidget(self.frame_reduce)
        self.frame_dark_Image = QtWidgets.QFrame(self.gb_ext_settings)
        self.frame_dark_Image.setObjectName("frame_dark_Image")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame_dark_Image)
        self.gridLayout_2.setContentsMargins(1, 1, 1, 1)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.chk_dark_image = QtWidgets.QCheckBox(self.frame_dark_Image)
        self.chk_dark_image.setObjectName("chk_dark_image")
        self.gridLayout_2.addWidget(self.chk_dark_image, 0, 0, 1, 1)
        self.but_acq_dark_image = QtWidgets.QPushButton(self.frame_dark_Image)
        self.but_acq_dark_image.setObjectName("but_acq_dark_image")
        self.gridLayout_2.addWidget(self.but_acq_dark_image, 1, 0, 1, 1)
        self.but_load_dark_image = QtWidgets.QPushButton(self.frame_dark_Image)
        self.but_load_dark_image.setObjectName("but_load_dark_image")
        self.gridLayout_2.addWidget(self.but_load_dark_image, 1, 1, 1, 1)
        self.but_save_dark_image = QtWidgets.QPushButton(self.frame_dark_Image)
        self.but_save_dark_image.setObjectName("but_save_dark_image")
        self.gridLayout_2.addWidget(self.but_save_dark_image, 1, 2, 1, 1)
        self.verticalLayout.addWidget(self.frame_dark_Image)
        self.frame_FPS = QtWidgets.QFrame(self.gb_ext_settings)
        self.frame_FPS.setObjectName("frame_FPS")
        self.layout_FPS = QtWidgets.QHBoxLayout(self.frame_FPS)
        self.layout_FPS.setObjectName("layout_FPS")
        self.lbFps = QtWidgets.QLabel(self.frame_FPS)
        self.lbFps.setMinimumSize(QtCore.QSize(125, 0))
        self.lbFps.setObjectName("lbFps")
        self.layout_FPS.addWidget(self.lbFps)
        self.sb_FPS = QtWidgets.QSpinBox(self.frame_FPS)
        self.sb_FPS.setMinimumSize(QtCore.QSize(75, 0))
        self.sb_FPS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sb_FPS.setMinimum(1)
        self.sb_FPS.setObjectName("sb_FPS")
        self.layout_FPS.addWidget(self.sb_FPS)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_FPS.addItem(spacerItem5)
        self.verticalLayout.addWidget(self.frame_FPS)
        self.frame_background = QtWidgets.QFrame(self.gb_ext_settings)
        self.frame_background.setObjectName("frame_background")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.frame_background)
        self.gridLayout_3.setContentsMargins(1, 1, 1, 1)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.chk_background = QtWidgets.QCheckBox(self.frame_background)
        self.chk_background.setObjectName("chk_background")
        self.gridLayout_3.addWidget(self.chk_background, 0, 0, 1, 2)
        self.label_6 = QtWidgets.QLabel(self.frame_background)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 0, 3, 1, 1)
        self.dsb_sigmas = QtWidgets.QDoubleSpinBox(self.frame_background)
        self.dsb_sigmas.setObjectName("dsb_sigmas")
        self.gridLayout_3.addWidget(self.dsb_sigmas, 0, 2, 1, 1)
        self.verticalLayout.addWidget(self.frame_background)
        self.gb_screen_motor = QtWidgets.QGroupBox(self.gb_ext_settings)
        self.gb_screen_motor.setObjectName("gb_screen_motor")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.gb_screen_motor)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lb_screen_status = QtWidgets.QLabel(self.gb_screen_motor)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lb_screen_status.setFont(font)
        self.lb_screen_status.setObjectName("lb_screen_status")
        self.horizontalLayout_2.addWidget(self.lb_screen_status)
        self.but_in_out = QtWidgets.QPushButton(self.gb_screen_motor)
        self.but_in_out.setMinimumSize(QtCore.QSize(50, 0))
        self.but_in_out.setObjectName("but_in_out")
        self.horizontalLayout_2.addWidget(self.but_in_out)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem6)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem7)
        self.chk_auto_screen = QtWidgets.QCheckBox(self.gb_screen_motor)
        self.chk_auto_screen.setObjectName("chk_auto_screen")
        self.horizontalLayout_2.addWidget(self.chk_auto_screen)
        self.verticalLayout.addWidget(self.gb_screen_motor)
        self.tbAllParams = QtWidgets.QPushButton(self.gb_ext_settings)
        self.tbAllParams.setObjectName("tbAllParams")
        self.verticalLayout.addWidget(self.tbAllParams)
        self.verticalLayout_2.addWidget(self.gb_ext_settings)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem8)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem9)

        self.retranslateUi(SettingsWidget)
        QtCore.QMetaObject.connectSlotsByName(SettingsWidget)

    def retranslateUi(self, SettingsWidget):
        _translate = QtCore.QCoreApplication.translate
        SettingsWidget.setWindowTitle(_translate("SettingsWidget", "Form"))
        self.groupBox.setTitle(_translate("SettingsWidget", "Image settings"))
        self.label_26.setText(_translate("SettingsWidget", "Source:"))
        self.label_23.setText(_translate("SettingsWidget", "Folder to monitor:"))
        self.label.setText(_translate("SettingsWidget", "Exposure:"))
        self.label_3.setText(_translate("SettingsWidget", "Gain:"))
        self.groupBox_4.setTitle(_translate("SettingsWidget", "Levels"))
        self.chk_auto_levels.setText(_translate("SettingsWidget", "Auto Levels"))
        self.label_2.setText(_translate("SettingsWidget", "Min:"))
        self.label_4.setText(_translate("SettingsWidget", "Max:"))
        self.rb_lin_level.setText(_translate("SettingsWidget", "Lin"))
        self.rb_log_level.setText(_translate("SettingsWidget", "Log"))
        self.rb_sqrt_level.setText(_translate("SettingsWidget", "Sqrt"))
        self.chk_additional_settings.setText(_translate("SettingsWidget", "Show additional settings"))
        self.gb_ext_settings.setTitle(_translate("SettingsWidget", "Additional settings"))
        self.label_22.setText(_translate("SettingsWidget", "Cut image:"))
        self.label_19.setText(_translate("SettingsWidget", "X:"))
        self.label_28.setText(_translate("SettingsWidget", "H:"))
        self.label_20.setText(_translate("SettingsWidget", "Y:"))
        self.label_27.setText(_translate("SettingsWidget", "W:"))
        self.lbFps_2.setText(_translate("SettingsWidget", "Reduce resolution"))
        self.lbFps_3.setText(_translate("SettingsWidget", "times"))
        self.chk_dark_image.setText(_translate("SettingsWidget", "Substract dark image"))
        self.but_acq_dark_image.setText(_translate("SettingsWidget", "Acquire Dark Image"))
        self.but_load_dark_image.setText(_translate("SettingsWidget", "Load Dark Image"))
        self.but_save_dark_image.setText(_translate("SettingsWidget", "Save Dark Image"))
        self.lbFps.setText(_translate("SettingsWidget", "Limit FPS"))
        self.chk_background.setText(_translate("SettingsWidget", "Substract mean background:"))
        self.label_6.setText(_translate("SettingsWidget", "sigmas"))
        self.gb_screen_motor.setTitle(_translate("SettingsWidget", "Screen control"))
        self.lb_screen_status.setText(_translate("SettingsWidget", "Screen In"))
        self.but_in_out.setText(_translate("SettingsWidget", "Move In"))
        self.chk_auto_screen.setText(_translate("SettingsWidget", "Enable auto screen"))
        self.tbAllParams.setText(_translate("SettingsWidget", "Open ATK panel"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/PeakSearch.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PeakSearch(object):
    def setupUi(self, PeakSearch):
        PeakSearch.setObjectName("PeakSearch")
        PeakSearch.resize(472, 869)
        self.verticalLayout = QtWidgets.QVBoxLayout(PeakSearch)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.chk_peak_search = QtWidgets.QCheckBox(PeakSearch)
        self.chk_peak_search.setObjectName("chk_peak_search")
        self.horizontalLayout_8.addWidget(self.chk_peak_search)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.rb_abs_threshold = QtWidgets.QRadioButton(PeakSearch)
        self.rb_abs_threshold.setChecked(True)
        self.rb_abs_threshold.setObjectName("rb_abs_threshold")
        self.horizontalLayout_10.addWidget(self.rb_abs_threshold)
        self.sl_abs_threshold = QtWidgets.QSlider(PeakSearch)
        self.sl_abs_threshold.setMinimum(1)
        self.sl_abs_threshold.setMaximum(16000)
        self.sl_abs_threshold.setSingleStep(1000)
        self.sl_abs_threshold.setOrientation(QtCore.Qt.Horizontal)
        self.sl_abs_threshold.setTickPosition(QtWidgets.QSlider.TicksBelow)
        self.sl_abs_threshold.setTickInterval(500)
        self.sl_abs_threshold.setObjectName("sl_abs_threshold")
        self.horizontalLayout_10.addWidget(self.sl_abs_threshold)
        self.sb_abs_threshold = QtWidgets.QSpinBox(PeakSearch)
        self.sb_abs_threshold.setMinimum(1)
        self.sb_abs_threshold.setMaximum(16000)
        self.sb_abs_threshold.setSingleStep(1000)
        self.sb_abs_threshold.setObjectName("sb_abs_threshold")
        self.horizontalLayout_10.addWidget(self.sb_abs_threshold)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.rb_rel_threshold = QtWidgets.QRadioButton(PeakSearch)
        self.rb_rel_threshold.setChecked(False)
        self.rb_rel_threshold.setObjectName("rb_rel_threshold")
        self.horizontalLayout_7.addWidget(self.rb_rel_threshold)
        self.sl_rel_threshold = QtWidgets.QSlider(PeakSearch)
        self.sl_rel_threshold.setEnabled(False)
        self.sl_rel_threshold.setMinimum(1)
        self.sl_rel_threshold.setSingleStep(10)
        self.sl_rel_threshold.setOrientation(QtCore.Qt.Horizontal)
        self.sl_rel_threshold.setTickPosition(QtWidgets.QSlider.TicksBelow)
        self.sl_rel_threshold.setTickInterval(10)
        self.sl_rel_threshold.setObjectName("sl_rel_threshold")
        self.horizontalLayout_7.addWidget(self.sl_rel_threshold)
        self.sb_rel_threshold = QtWidgets.QSpinBox(PeakSearch)
        self.sb_rel_threshold.setEnabled(False)
        self.sb_rel_threshold.setMinimum(1)
        self.sb_rel_threshold.setSingleStep(10)
        self.sb_rel_threshold.setObjectName("sb_rel_threshold")
        self.horizontalLayout_7.addWidget(self.sb_rel_threshold)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        spacerItem1 = QtWidgets.QSpacerItem(412, 750, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(PeakSearch)
        QtCore.QMetaObject.connectSlotsByName(PeakSearch)

    def retranslateUi(self, PeakSearch):
        _translate = QtCore.QCoreApplication.translate
        PeakSearch.setWindowTitle(_translate("PeakSearch", "Peak Search"))
        self.chk_peak_search.setText(_translate("PeakSearch", "Enable peak search"))
        self.rb_abs_threshold.setText(_translate("PeakSearch", "Absolute threshold"))
        self.rb_rel_threshold.setText(_translate("PeakSearch", "Relative thershold"))




# Created by matveyev at 19.08.2021

from PyQt5 import QtCore

from petra_camera.widgets.base_widget import BaseWidget

from petra_camera.gui.PeakSearch_ui import Ui_PeakSearch

WIDGET_NAME = 'PeakSearch'


# ----------------------------------------------------------------------
class PeakSearchWidget(BaseWidget):

    refresh_image = QtCore.pyqtSignal()

    # ----------------------------------------------------------------------
    def __init__(self, parent):

        super(PeakSearchWidget, self).__init__(parent)

        self._ui = Ui_PeakSearch()
        self._ui.setupUi(self)

        self._load_camera_settings()

        self._ui.chk_peak_search.clicked.connect(lambda: self._peak_search_modified('chk_peak_search'))
        self._ui.rb_abs_threshold.clicked.connect(lambda: self._peak_search_modified('mode'))
        self._ui.rb_rel_threshold.clicked.connect(lambda: self._peak_search_modified('mode'))
        self._ui.sl_rel_threshold.valueChanged.connect(lambda: self._peak_search_modified('sl_rel'))
        self._ui.sb_rel_threshold.editingFinished.connect(lambda: self._peak_search_modified('sb_rel'))
        self._ui.sl_abs_threshold.valueChanged.connect(lambda: self._peak_search_modified('sl_abs'))
        self._ui.sb_abs_threshold.editingFinished.connect(lambda: self._peak_search_modified('sb_abs'))

    # ----------------------------------------------------------------------
    def _load_camera_settings(self):

        self._ui.chk_peak_search.setChecked(self._camera_device.peak_search['search'])
        self._ui.rb_rel_threshold.setChecked(self._camera_device.peak_search['search_mode'])
        self._ui.rb_abs_threshold.setChecked(not self._camera_device.peak_search['search_mode'])
        self._ui.sl_rel_threshold.setValue(self._camera_device.peak_search['rel_threshold'])
        self._ui.sb_rel_threshold.setValue(self._camera_device.peak_search['rel_threshold'])
        self._ui.sl_abs_threshold.setValue(self._camera_device.peak_search['abs_threshold'])
        self._ui.sb_abs_threshold.setValue(self._camera_device.peak_search['abs_threshold'])

    # ----------------------------------------------------------------------
    def _peak_search_modified(self, ui):

        state = self._ui.chk_peak_search.isChecked()
        self._camera_device.set_peak_search_value('search', state)
        mode = self._ui.rb_rel_threshold.isChecked()
        self._camera_device.set_peak_search_value('search_mode', mode)

        if ui == 'sl_abs':
            threshold = self._ui.sl_abs_threshold.value()
            self._ui.sb_abs_threshold.blockSignals(True)
            self._ui.sb_abs_threshold.setValue(threshold)
            self._ui.sb_abs_threshold.blockSignals(False)
        elif ui == 'sb_abs':
            threshold = self._ui.sb_abs_threshold.value()
            self._ui.sl_abs_threshold.blockSignals(True)
            self._ui.sl_abs_threshold.setValue(threshold)
            self._ui.sl_abs_threshold.blockSignals(False)
        elif ui == 'sl_rel':
            threshold = self._ui.sl_rel_threshold.value()
            self._ui.sl_rel_threshold.blockSignals(True)
            self._ui.sl_rel_threshold.setValue(threshold)
            self._ui.sl_rel_threshold.blockSignals(False)
        elif ui == 'sb_rel':
            threshold = self._ui.sb_rel_threshold.value()
            self._ui.sl_rel_threshold.blockSignals(True)
            self._ui.sl_rel_threshold.setValue(threshold)
            self._ui.sl_rel_threshold.blockSignals(False)

        if mode:
            threshold = self._ui.sl_rel_threshold.value()
            self._camera_device.set_peak_search_value('rel_threshold', threshold)
            self._ui.sl_abs_threshold.setEnabled(False)
            self._ui.sb_abs_threshold.setEnabled(False)
            self._ui.sl_rel_threshold.setEnabled(True)
            self._ui.sb_rel_threshold.setEnabled(True)
        else:
            threshold = self._ui.sl_abs_threshold.value()
            self._camera_device.set_peak_search_value('abs_threshold', threshold)
            self._ui.sl_abs_threshold.setEnabled(True)
            self._ui.sb_abs_threshold.setEnabled(True)
            self._ui.sl_rel_threshold.setEnabled(False)
            self._ui.sb_rel_threshold.setEnabled(False)

        self.refresh_image.emit()

    # ----------------------------------------------------------------------
    def _block_signals(self, flag):

        self._ui.sl_rel_threshold.blockSignals(flag)
        self._ui.sb_rel_threshold.blockSignals(flag)
        self._ui.sl_abs_threshold.blockSignals(flag)
        self._ui.sb_abs_threshold.blockSignals(flag)
        self._ui.rb_abs_threshold.blockSignals(flag)
        self._ui.rb_rel_threshold.blockSignals(flag)
        self._ui.chk_peak_search.blockSignals(flag)


# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""
Base camera class
"""

import PyTango
import logging
import numpy as np

from PyQt5 import QtCore
from distutils.util import strtobool

from petra_camera.devices.screen_motor import MotorExecutor

from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class BaseCamera(object):

    # ----------------------------------------------------------------------
    def __init__(self, settings):
        """

        :param settings: config.xml
        :param log:
        """
        super(BaseCamera, self).__init__()

        self._settings = settings

        self.file_name = ''

        self._new_frame_flag = False
        self._eid = None  # Tango even ID

        self._my_name = settings.getAttribute("name")

        # picture rotate and flip properties
        if settings.hasAttribute('flip_vertical'):
            self.flip_v = bool(strtobool(settings.getAttribute("flip_vertical")))
        else:
            self.flip_v = False

        if settings.hasAttribute('flip_horizontal'):
            self.flip_h = bool(strtobool(settings.getAttribute("flip_horizontal")))
        else:
            self.flip_h = False

        if settings.hasAttribute('rotate'):
            self.rotate_angle = int(settings.getAttribute("rotate"))
        else:
            self.rotate_angle = 0

        # DeviceProxies instances
        if settings.hasAttribute('tango_server'):
            name = str(settings.getAttribute("tango_server"))
            self._device_proxy = PyTango.DeviceProxy(name)
            if self._device_proxy.state() == PyTango.DevState.FAULT:
                raise RuntimeError(f'{name} in FAULT state!')
            logger.debug(f'{self._my_name}: new tango proxy {name}')
        else:
            self._device_proxy = None

        if settings.hasAttribute('settings_server'):
            name = str(settings.getAttribute("settings_server"))
            self._settings_proxy = PyTango.DeviceProxy(name)
            if self._settings_proxy.state() == PyTango.DevState.FAULT:
                raise RuntimeError(f'{name} in FAULT state!')
            logger.debug(f'{self._my_name}: new settings proxy {name}')
        else:
            self._settings_proxy = None

        if settings.hasAttribute('roi_server'):
            name = str(settings.getAttribute("roi_server"))
            self._roi_server = PyTango.DeviceProxy(name)
            if self._roi_server.state() == PyTango.DevState.FAULT:
                raise RuntimeError(f'{name} in FAULT state!')
            logger.debug(f'{self._my_name}: new roi server {name}')
        else:
            self._roi_server = None

        # if camera has a screen control
        if settings.hasAttribute('motor_type') and \
                (str(settings.getAttribute("motor_type")).lower() not in ['none', 'no', '']):
            self._motor_worker = MotorExecutor(settings)
        else:
            self._motor_worker = None

        # for high resolution cameras to decrease CPU load
        self.reduce_resolution = max(self.get_settings('Reduce', int), 1)

        self._picture_size = [0, 0, -1, -1]

    # ----------------------------------------------------------------------
    def close_camera(self):
        """
        save camera close
        :return:
        """
        if self._motor_worker is not None:
            self._motor_worker.stop()

    # ----------------------------------------------------------------------
    def start_acquisition(self):
        raise RuntimeError('Not implemented')

    # ----------------------------------------------------------------------
    def stop_acquisition(self):
        raise RuntimeError('Not implemented')

    # ----------------------------------------------------------------------
    # ------------------------ Frame functionality -------------------------
    # ----------------------------------------------------------------------
    def maybe_read_frame(self):
        """

        :return: None if no new picture or 2d np.array if there is a new frame
        """
        if not self._new_frame_flag:
            return None

        self._new_frame_flag = False
        return self.rotate()

    # ----------------------------------------------------------------------
    def rotate(self):
        """
        rotate and flip picture
        :return: 2d np.array
        """

        if self.flip_v and self.flip_h:
            self._last_frame = self._last_frame[::-1, ::-1]
        elif self.flip_v:
            self._last_frame = self._last_frame[::, ::-1]
        elif self.flip_h:
            self._last_frame = self._last_frame[::-1, :]

        if self.rotate_angle:
            self._last_frame = np.rot90(self._last_frame, self.rotate_angle)

        return self._last_frame[::self.reduce_resolution, ::self.reduce_resolution]

    # ----------------------------------------------------------------------
    def is_running(self):
        """

        :return: bool
        """
        return True

    # ----------------------------------------------------------------------
    # ------------------------- Picture clip/resolution  -------------------
    # ----------------------------------------------------------------------
    def set_picture_clip(self, size):
        """
        sets picture clip

        :param size: (x, y, w, h) - parameters of picture clip
        :return: None
        """

        self._picture_size = [size[0], size[1], size[0]+size[2], size[1]+size[3]]

        self.save_settings('view_x', size[0])
        self.save_settings('view_y', size[1])
        self.save_settings('view_w', size[2])
        self.save_settings('view_h', size[3])

    # ----------------------------------------------------------------------
    def get_picture_clip(self):
        """

        :return: (x, y, w, h) - parameters of picture clip
        """

        return [self._picture_size[0], self._picture_size[1],
                self._picture_size[2] - self._picture_size[0],
                self._picture_size[3] - self._picture_size[1]]

    # ----------------------------------------------------------------------
    def get_reduction(self):
        """

        :return: int, picture reduction
        """
        return self.reduce_resolution

    # ----------------------------------------------------------------------
    def set_reduction(self, value):
        """
        sets picture reduction

        :param value: int,
        :return:
        """
        self.save_settings('Reduce', value)
        self.reduce_resolution = value

    # ----------------------------------------------------------------------
    # ------------------------ Camera settings load/save -------------------
    # ----------------------------------------------------------------------
    def get_settings(self, option, cast):
        """
         reads the requested setting according the settings map

        :param option: str, setting name
        :param cast:
        :return: None, of False (if cast == bool), '' (is cast == str) if there is no such settings,
                 or cast(requested setting)
        """

        logger.debug(f'{self._my_name}: setting {cast.__name__}({option}) requested')

        if option in self._settings_map.keys():
            try:
                if self._settings_map[option][0] == 'roi_server' and self._roi_server is not None:
                    value = getattr(self._roi_server, self._settings_map[option][1])

                elif self._settings_map[option][0] == 'settings_proxy' and self._settings_proxy is not None:
                    value = self._settings_proxy.read_attribute(self._settings_map[option][1]).value

                elif self._settings_map[option][0] == 'device_proxy' and self._device_proxy is not None:
                    value = self._device_proxy.read_attribute(self._settings_map[option][1]).value

                elif self._settings_map[option][0] == 'self':
                    value = getattr(self, self._settings_map[option][1])

                elif self._settings_map[option][0] is None:
                    value = None
                else:
                    raise RuntimeError('Unknown setting source')
            except:
                value = None

        else:
            try:
                value = QtCore.QSettings(APP_NAME).value("{}/{}".format(self._my_name, option))
            except:
                value = None

        if option in ['view_w', 'view_h']:
            if value is None or int(value) < 1:
                if option == 'view_w':
                    return self.get_settings('max_width', int) - self.get_settings('view_x', int)
                else:
                    return self.get_settings('max_height', int) - self.get_settings('view_y', int)

        if value is not None:
            if cast == bool:
                try:
                    return strtobool(str(value))
                except:
                    print('Cannot convert settings {} {} to bool'.format(option, value))
                    return False
            elif cast == int:
                return int(float(value))
            else:
                return cast(value)

        else:
            if cast == bool:
                return False
            elif cast in [int, float]:
                return cast(0)
            elif cast == str:
                return ''
            else:
                return None

    # ----------------------------------------------------------------------
    def save_settings(self, option, value):
        """
        saves the requested setting according the settings map

        :param option: str, setting name
        :param value: new vale to save
        :return:
        """

        logger.debug(f'{self._my_name}: setting {option}: new value {value}')

        if option in self._settings_map.keys():
            if self._settings_map[option][0] == 'roi_server' and self._roi_server is not None:
                setattr(self._roi_server, self._settings_map[option][1], value)

            elif self._settings_map[option][0] == 'settings_proxy' and self._settings_proxy is not None:
                self._settings_proxy.write_attribute(self._settings_map[option][1], value)

            elif self._settings_map[option][0] == 'device_proxy' and self._device_proxy is not None:
                self._device_proxy.write_attribute(self._settings_map[option][1], value)

            elif self._settings_map[option][0] is None:
                pass

            else:
                raise RuntimeError('Unknown setting source')
        else:
            QtCore.QSettings(APP_NAME).setValue("{}/{}".format(self._my_name, option), value)

        if option == 'Reduce':
            self.reduce_resolution = value

    # ----------------------------------------------------------------------
    # ------------------------ Screen control ------------------------------
    # ----------------------------------------------------------------------
    def has_motor(self):
        """

        :return: bool
        """
        return self._motor_worker is not None

    # ----------------------------------------------------------------------
    def move_motor(self, state=None):
        """

        :param state:
        :return:
        """
        if self._motor_worker is not None:
            if state is None:
                state = not self.motor_position()

            self._motor_worker.move_motor(state)

    # ----------------------------------------------------------------------
    def motor_position(self):
        """

        :return: bool: True - screen in, False - screen out, None is there is no motor
        """
        if self._motor_worker is not None:
            return self._motor_worker.motor_position()
        else:
            return None

    # ----------------------------------------------------------------------
    # ------------------------ Sardana counter control ---------------------
    # ----------------------------------------------------------------------
    def has_counter(self):
        """

        :return: bool
        """
        return self._roi_server is not None

    # ----------------------------------------------------------------------
    def get_counter(self):
        """

        :return: str, current counter control
        """
        if self._roi_server is not None:
            try:
                return self._roi_server.scan_parameter
            except:
                return ''
        else:
            return ''

    # ----------------------------------------------------------------------
    def set_counter(self, value):
        """

        :param value: parameter to be set as a counter for Sardana control
        :return:
        """
        if self._roi_server is not None:
            try:
                self._roi_server.scan_parameter = str(value)
            except:
                pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/FrameViewer.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_FrameViewer(object):
    def setupUi(self, FrameViewer):
        FrameViewer.setObjectName("FrameViewer")
        FrameViewer.resize(945, 707)
        self.horizontalLayout = QtWidgets.QHBoxLayout(FrameViewer)
        self.horizontalLayout.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.splitter_x = QtWidgets.QSplitter(FrameViewer)
        self.splitter_x.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_x.setObjectName("splitter_x")
        self.splitter_y1 = QtWidgets.QSplitter(self.splitter_x)
        self.splitter_y1.setOrientation(QtCore.Qt.Vertical)
        self.splitter_y1.setObjectName("splitter_y1")
        self.image_view = ImageViewNoKeyboard(self.splitter_y1)
        self.image_view.setObjectName("image_view")
        self.wiProfileX = ProjectionWidget(self.splitter_y1)
        self.wiProfileX.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.wiProfileX.setObjectName("wiProfileX")
        self.splitter_y2 = QtWidgets.QSplitter(self.splitter_x)
        self.splitter_y2.setOrientation(QtCore.Qt.Vertical)
        self.splitter_y2.setObjectName("splitter_y2")
        self.wiProfileY = ProjectionWidget(self.splitter_y2)
        self.wiProfileY.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.wiProfileY.setObjectName("wiProfileY")
        self.widget = QtWidgets.QWidget(self.splitter_y2)
        self.widget.setMinimumSize(QtCore.QSize(200, 0))
        self.widget.setMaximumSize(QtCore.QSize(200, 200))
        self.widget.setObjectName("widget")
        self.horizontalLayout.addWidget(self.splitter_x)

        self.retranslateUi(FrameViewer)
        QtCore.QMetaObject.connectSlotsByName(FrameViewer)

    def retranslateUi(self, FrameViewer):
        _translate = QtCore.QCoreApplication.translate
        FrameViewer.setWindowTitle(_translate("FrameViewer", "Form"))

from petra_camera.widgets.image_view import ImageViewNoKeyboard
from petra_camera.widgets.projection_widget import ProjectionWidget
import petra_camera.gui.icons_rc



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""
"""

from PyQt5 import QtWidgets, QtCore
from petra_camera.gui.Marker_ui import Ui_Marker

# ----------------------------------------------------------------------
class Marker(QtWidgets.QWidget):

    repaint_marker = QtCore.pyqtSignal()
    delete_me = QtCore.pyqtSignal(int)

    # ----------------------------------------------------------------------
    def __init__(self, id, camera_device):

        super(Marker, self).__init__()

        self._ui = Ui_Marker()
        self._ui.setupUi(self)

        self._camera_device = camera_device
        self.my_id = id

        self._ui.but_delete.clicked.connect(lambda status, x=id: self.delete_me.emit(x))

        self._ui.sb_x.valueChanged.connect(lambda value: self.save_value('x', value))
        self._ui.sb_y.valueChanged.connect(lambda value: self.save_value('y', value))
        self._ui.chk_visible.clicked.connect(lambda value: self.save_value('visible', value))
        self._ui.but_color.clicked.connect(self._pick_my_color)

    # ----------------------------------------------------------------------
    def _block_signals(self, flag):
        self._ui.sb_x.blockSignals(flag)
        self._ui.sb_y.blockSignals(flag)
        self._ui.chk_visible.blockSignals(flag)

    # ----------------------------------------------------------------------
    def update_values(self):
        self._block_signals(True)
        self._ui.sb_x.setValue(self._camera_device.markers[self.my_id]['x'])
        self._ui.sb_y.setValue(self._camera_device.markers[self.my_id]['y'])
        self._ui.chk_visible.setChecked(self._camera_device.markers[self.my_id]['visible'])

        self._ui.but_color.setStyleSheet("QPushButton {background-color: " +
                                         f"{self._camera_device.markers[self.my_id]['color']}" + ";}")
        self._block_signals(False)

    # ----------------------------------------------------------------------
    def save_value(self, param, value):
        self._camera_device.set_marker_value(self.my_id, param, value)
        self.repaint_marker.emit()

    # ----------------------------------------------------------------------
    def _pick_my_color(self):
        color = QtWidgets.QColorDialog.getColor()

        if color.isValid():
            self._camera_device.set_marker_value(self.my_id, 'color', color.name())
            self.repaint_marker.emit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Author:        sebastian.piec@mail.desy.de
# Last modified: 2017, November 7
# ----------------------------------------------------------------------

"""Basic info dialog
"""

from datetime import datetime
import os

from PyQt5 import QtWidgets

from petra_camera.gui.AboutDialog_ui import Ui_AboutDialog
from petra_camera.version import __version__

# ----------------------------------------------------------------------
class AboutDialog(QtWidgets.QDialog):
    """
    """
    SOURCE_DIR = "petra_camera"
    DATETIME = "%Y-%m-%d %H:%M:%S"

    # ----------------------------------------------------------------------
    def __init__(self, parent):
        super(AboutDialog, self).__init__(parent)

        self._ui = Ui_AboutDialog()
        self._ui.setupUi(self)

        self._getModification()

    # ----------------------------------------------------------------------
    def _getModification(self):
        """Display last source code modification date.
        """
        mtime = 0

        for root, _, files in os.walk(self.SOURCE_DIR, topdown=True):
            for name in files:
                filename = os.path.join(root, name)
                _, ext = os.path.splitext(filename)

                if ext == ".py":
                    ftime = os.path.getmtime(filename)
                    if ftime > mtime:
                        mtime = ftime

        txt = datetime.fromtimestamp(mtime).strftime(self.DATETIME)
        self._ui.lbModified.setText(f"Last modification: {txt}")
        try:
            self._ui.label_4.setText(f"Version: {__version__}")
        except:
            pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/AboutDialog.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(388, 195)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AboutDialog.sizePolicy().hasHeightForWidth())
        AboutDialog.setSizePolicy(sizePolicy)
        AboutDialog.setMinimumSize(QtCore.QSize(330, 195))
        AboutDialog.setMaximumSize(QtCore.QSize(450, 195))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lbLogo = QtWidgets.QLabel(AboutDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbLogo.sizePolicy().hasHeightForWidth())
        self.lbLogo.setSizePolicy(sizePolicy)
        self.lbLogo.setMinimumSize(QtCore.QSize(130, 0))
        self.lbLogo.setText("")
        self.lbLogo.setPixmap(QtGui.QPixmap(":/ico/Desy_logo.jpg"))
        self.lbLogo.setObjectName("lbLogo")
        self.horizontalLayout_2.addWidget(self.lbLogo)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_3 = QtWidgets.QLabel(AboutDialog)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.label_4 = QtWidgets.QLabel(AboutDialog)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.lbModified = QtWidgets.QLabel(AboutDialog)
        self.lbModified.setText("")
        self.lbModified.setObjectName("lbModified")
        self.verticalLayout.addWidget(self.lbModified)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.lbEmail = QtWidgets.QLabel(AboutDialog)
        self.lbEmail.setMinimumSize(QtCore.QSize(200, 0))
        self.lbEmail.setObjectName("lbEmail")
        self.verticalLayout.addWidget(self.lbEmail)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(AboutDialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButton = QtWidgets.QPushButton(AboutDialog)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout.addWidget(self.pushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(AboutDialog)
        self.pushButton.clicked.connect(AboutDialog.accept)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_translate("AboutDialog", "Camera viewer"))
        self.label_3.setText(_translate("AboutDialog", "2D Data Viewer"))
        self.label_4.setText(_translate("AboutDialog", "Version: 2.0"))
        self.lbEmail.setText(_translate("AboutDialog", "yury.matveev@desy.de"))
        self.pushButton.setText(_translate("AboutDialog", "OK"))

import petra_camera.gui.icons_rc



# Created by matveyev at 27.08.2021
import pyqtgraph as pg
from PyQt5 import QtGui, QtCore

from petra_camera.utils.functions import rotate


# ----------------------------------------------------------------------
class ImageMarker(QtCore.QObject):
    """Infinite lines cross
    """

    new_coordinates = QtCore.pyqtSignal(int, int)

    # ----------------------------------------------------------------------
    def __init__(self, x, y, image_view):
        super(ImageMarker, self).__init__()

        self.image_view = image_view

        self._markerV = pg.InfiniteLine(pos=x, movable=True)
        self._markerV.sigPositionChanged.connect(self._new_coordinates)
        self.image_view.addItem(self._markerV, ignoreBounds=True)

        self._markerH = pg.InfiniteLine(pos=y, angle=0, movable=True)
        self._markerH.sigPositionChanged.connect(self._new_coordinates)
        self.image_view.addItem(self._markerH, ignoreBounds=True)

    # ----------------------------------------------------------------------
    def _new_coordinates(self):
        self.new_coordinates.emit(int(self._markerV.pos().x()), int(self._markerH.pos().y()))

    # ----------------------------------------------------------------------
    def setPos(self, x, y):
        """
        """

        self._markerV.sigPositionChanged.disconnect()
        self._markerH.sigPositionChanged.disconnect()
        self._markerV.setPos(x)
        self._markerH.setPos(y)
        self._markerV.sigPositionChanged.connect(self._new_coordinates)
        self._markerH.sigPositionChanged.connect(self._new_coordinates)

    # ----------------------------------------------------------------------
    def setVisible(self, flag):
        """
        """
        self._markerV.setVisible(flag)
        self._markerH.setVisible(flag)

    # ----------------------------------------------------------------------
    def visible(self):
        """
        """
        return self._markerV.isVisible() and self._markerH.isVisible()

    # ----------------------------------------------------------------------
    def delete_me(self):

        self.image_view.removeItem(self._markerH)
        self.image_view.removeItem(self._markerV)

    # ----------------------------------------------------------------------
    def setColor(self, color):

        self._markerH.setPen(pg.mkPen(color))
        self._markerV.setPen(pg.mkPen(color))


# ----------------------------------------------------------------------
class PeakMarker(pg.GraphicsObject):
    """
        Circle object
    """

    # ----------------------------------------------------------------------
    def __init__(self):
        super(PeakMarker, self).__init__()
        pg.GraphicsObject.__init__(self)
        self._picture = QtGui.QPicture()
        self._positions = ()
        self._size = 0

    # ----------------------------------------------------------------------
    def new_peaks(self, positions):
        self._positions = positions
        self._draw()

    # ----------------------------------------------------------------------
    def new_scale(self, picture_w, picture_h):
        self._size = int(0.01*min(picture_w, picture_h))
        self._draw()

    # ----------------------------------------------------------------------
    def _draw(self):

        p = QtGui.QPainter(self._picture)
        p.setPen(pg.mkPen('r', width=2))

        for x, y in self._positions:
            p.drawEllipse(QtCore.QPoint(x, y), self._size, self._size)

        p.end()

    # ----------------------------------------------------------------------
    def paint(self, p, *args):
        p.drawPicture(0, 0, self._picture)

    # ----------------------------------------------------------------------
    def boundingRect(self):
        return QtCore.QRectF(self._picture.boundingRect())


# ----------------------------------------------------------------------
class LineSegmentItem(pg.GraphicsObject):

    def __init__(self, mode, cross_size=1., circle=0.):
        pg.GraphicsObject.__init__(self)
        self._mode = mode
        self._picture = QtGui.QPicture()

        self._scaled_circle_size = 0

        self._line1_end1 = QtCore.QPoint(0, 0)
        self._line1_end2 = QtCore.QPoint(0, 0)
        self._line2_end1 = QtCore.QPoint(0, 0)
        self._line2_end2 = QtCore.QPoint(0, 0)

        self._draw_lines = False
        self._draw_point1 = False
        self._draw_point2 = False

        self._cross_size = cross_size/2
        self._circle_size = circle

        self.generate_picture()

    # ----------------------------------------------------------------------
    def set_pos(self, *argin):
        """

        :param argin:
        :return:
        """
        if self._mode == 'cross':
            # here we get argin[0] - center position
            # here we get argin[1] - line length

            if not (None in argin[0] or None in argin[1]):
                self._line1_end1 = QtCore.QPoint(int(argin[0][0] - argin[1][0]/2), int(argin[0][1]))
                self._line1_end2 = QtCore.QPoint(int(argin[0][0] + argin[1][0]/2), int(argin[0][1]))

                self._line2_end1 = QtCore.QPoint(int(argin[0][0]), int(argin[0][1] - argin[1][1]/2))
                self._line2_end2 = QtCore.QPoint(int(argin[0][0]), int(argin[0][1] + argin[1][1]/2))

                self._draw_lines = True
                self._draw_point1 = False
                self._draw_point2 = False

        else:

            self._draw_lines = True
            self._draw_point1 = False
            self._draw_point2 = False

            if argin[0][0] is not None:
                self._line1_end1 = argin[0][0]
                self._draw_point1 = True
            else:
                self._draw_lines = False

            if argin[0][1] is not None:
                self._line1_end2 = argin[0][1]
                self._draw_point2 = True
            else:
                self._draw_lines = False

            if self._draw_lines:
                center = ((self._line1_end1.x() + self._line1_end2.x()) / 2,
                          (self._line1_end1.y() + self._line1_end2.y()) / 2)

                point = (self._line1_end1.x() * (0.5 + self._cross_size) + self._line1_end2.x() * (0.5 - self._cross_size),
                         self._line1_end1.y() * (0.5 + self._cross_size) + self._line1_end2.y() * (0.5 - self._cross_size))

                p1 = rotate(center, point, 1.57)
                p2 = rotate(center, point, -1.57)

                self._line2_end1 = QtCore.QPoint(p1[0], p1[1])
                self._line2_end2 = QtCore.QPoint(p2[0], p2[1])

        self.generate_picture()

    # ----------------------------------------------------------------------
    def new_scale(self, w, h):
        self._scaled_circle_size = int(self._circle_size*min(w, h))
        self.generate_picture()

    # ----------------------------------------------------------------------
    def generate_picture(self):

        p = QtGui.QPainter(self._picture)
        p.setPen(pg.mkPen('r', width=2, style=QtCore.Qt.DotLine))

        if self._draw_lines:
            # Horizontal
            p.drawLine(self._line1_end1, self._line1_end2)

            # Vertical
            p.drawLine(self._line2_end1, self._line2_end2)

        p.setPen(QtCore.Qt.NoPen)
        p.setBrush(pg.mkBrush('r'))

        if self._draw_point1:
            p.drawEllipse(self._line1_end1, self._scaled_circle_size, self._scaled_circle_size)

        if self._draw_point2:
            p.drawEllipse(self._line1_end2, self._scaled_circle_size, self._scaled_circle_size)

        p.end()

    # ----------------------------------------------------------------------
    def paint(self, p, *args):
        p.drawPicture(0, 0, self._picture)

    # ----------------------------------------------------------------------
    def boundingRect(self):
        return QtCore.QRectF(self._picture.boundingRect())


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/LogWidget.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_LogWidget(object):
    def setupUi(self, LogWidget):
        LogWidget.setObjectName("LogWidget")
        LogWidget.resize(561, 410)
        self.horizontalLayout = QtWidgets.QHBoxLayout(LogWidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.teLogs = QtWidgets.QPlainTextEdit(LogWidget)
        self.teLogs.setReadOnly(True)
        self.teLogs.setObjectName("teLogs")
        self.horizontalLayout.addWidget(self.teLogs)

        self.retranslateUi(LogWidget)
        QtCore.QMetaObject.connectSlotsByName(LogWidget)

    def retranslateUi(self, LogWidget):
        _translate = QtCore.QCoreApplication.translate
        LogWidget.setWindowTitle(_translate("LogWidget", "Form"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Author:        sebastian.piec@desy.de
# ----------------------------------------------------------------------

"""Parse settings given in xml files
"""

from xml.dom.minidom import parseString


# ----------------------------------------------------------------------
class XmlSettings(object):

    # ----------------------------------------------------------------------
    def __init__(self, fileName):
        self.fileName = fileName

    # ----------------------------------------------------------------------
    def option(self, nodePath, attribute):
        """Retrieve option's value from a config file.
        """
        with open(self.fileName, "r") as inFile:        # read over-and-over??? TODO
            path = nodePath.split("/")

            node = parseString(inFile.read())
            for nodeName in path:
                node = node.getElementsByTagName(nodeName)[0]     # take first node always...

            return node.getAttribute(attribute)

    # ----------------------------------------------------------------------
    def get_nodes(self, nodePath, nodeName):
        """
        """
        with open(self.fileName, "r") as inFile:
            path = nodePath.split("/")[:-1]

            node = parseString(inFile.read())
            for nodeName in path:
                node = node.getElementsByTagName(nodeName)[0]

            return node.getElementsByTagName(nodeName)

    # ----------------------------------------------------------------------
    def node(self, nodePath):
        """
        """
        with open(self.fileName, "r") as f:
            path = nodePath.split("/")

            node = parseString(f.read())
            for nodeName in path[:-1]:
                node = node.getElementsByTagName(nodeName)[0]

            return node.getElementsByTagName(path[-1])[0]           # returns first matching node always...

    # ----------------------------------------------------------------------
    def has_node(self, nodePath):
        try:
            self.node(nodePath)
            return True
        except:
            return False



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""Camera motor class
"""

import socket
import errno, time
import json
import logging
import PyTango
import threading

from io import StringIO
from queue import Queue, Empty

from petra_camera.main_window import APP_NAME

REFRESH_PERIOD = 1

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class MotorExecutor(object):

    SOCKET_TIMEOUT = 5
    DATA_BUFFER_SIZE = 2 ** 22

    def __init__(self, settings):
        super(MotorExecutor, self).__init__()

        self._my_name = settings.getAttribute("name")
        if str(settings.getAttribute("motor_type")).lower() == 'acromag':
            self._motor_type = 'Acromag'
            server_name = str(settings.getAttribute("valve_tango_server"))
            self._valve_device_proxy = PyTango.DeviceProxy(server_name)
            if self._valve_device_proxy.state() == PyTango.DevState.FAULT:
                raise RuntimeError(f'{server_name} in FAULT state!')
            self._valve_channel = int(settings.getAttribute("valve_channel"))

            logger.debug(f'{self._my_name}: new Acromag motor: {server_name}:{self._valve_channel}')

        elif str(settings.getAttribute("motor_type")).lower() == 'fsbt':
            self._motor_type = 'FSBT'
            self._motor_name = str(settings.getAttribute("motor_name"))

            self._fsbt_server = None
            self._fsbt_host = str(settings.getAttribute("motor_host"))
            self._fsbt_port = int(settings.getAttribute("motor_port"))

            self._fsbt_worker = threading.Thread(target=self.server_connection)
            self._fsbt_worker_status = 'running'
            self._run_server = True
            self._motor_position = None
            self._move_queue = Queue()

            self._fsbt_worker.start()

            logger.debug(f'{self._my_name}: new FSBT motor: {self._motor_name}@{self._fsbt_host}:{self._fsbt_port}')

        else:
            raise RuntimeError('Unknown type of motor')

    # ----------------------------------------------------------------------
    def server_connection(self):

        if not self.get_connection_to_fsbt():
            self._fsbt_worker_status = 'stopped'
            return

        while self._run_server:
            try:
                status = self.send_command_to_fsbt('status ' + self._motor_name)
                self._motor_position = status[1][self._motor_name] == 'in'
            except Exception as err:
                logger.error("Error during motor status {}...".format(err))
                if not self.get_connection_to_fsbt():
                    break

            try:
                result = self.send_command_to_fsbt(self._move_queue.get(block=False))
                if not result[0]:
                    logger.error("Cannot move motor")

            except Empty:
                pass

            except Exception as err:
                logger.error("Error during motor move {}...".format(err))
                break

            time.sleep(REFRESH_PERIOD)

        self._fsbt_worker_status = 'stopped'

    # ----------------------------------------------------------------------
    def stop(self):

        if self._motor_type == 'FSBT' and self._fsbt_worker_status != 'stopped':
            while not self._move_queue.empty() and self._fsbt_worker_status != 'stopped':
                logger.debug(f'{self._my_name}: need to finish motor command queue')
                time.sleep(0.1)

            self._run_server = False
            while self._fsbt_worker_status != 'stopped':
                time.sleep(0.1)

        logger.debug(f'{self._my_name}: motor worker stopped')

    # ----------------------------------------------------------------------
    def motor_position(self):

        if self._motor_type == 'Acromag':
            _currentPos = list('{0:04b}'.format(int(self._valve_device_proxy.read_attribute("Register0").value)))
            return _currentPos[3 - self._valve_channel] == "1"

        elif self._motor_type == 'FSBT':
            return self._motor_position

    # ----------------------------------------------------------------------
    def move_motor(self, new_state):

        logger.debug(f'{self._my_name}: new move command {new_state}')

        if self._motor_type == 'Acromag':
            _currentPos = list('{0:04b}'.format(int(self._valve_device_proxy.read_attribute("Register0").value)))
            current_state = _currentPos[3 - self._valve_channel] == "1"

            if current_state != new_state:
                if new_state:
                    _currentPos[3 - self._valve_channel] = "1"
                else:
                    _currentPos[3 - self._valve_channel] = "0"

                self._valve_device_proxy.write_attribute("Register0", int("".join(_currentPos), 2))

        elif self._motor_type == 'FSBT':
            if self._motor_position != new_state:
                if new_state:
                    self._move_queue.put('in {:s}'.format(self._motor_name))
                else:
                    self._move_queue.put('out {:s}'.format(self._motor_name))
        else:
            raise RuntimeError('Unknown type of motor')

    # ----------------------------------------------------------------------
    def get_connection_to_fsbt(self):

        self._fsbt_server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._fsbt_server.settimeout(self.SOCKET_TIMEOUT)

        start_timeout = time.time()
        time_out = False
        is_connected = False
        while not time_out and not is_connected:
            err = self._fsbt_server.connect_ex((self._fsbt_host, self._fsbt_port))
            if err == 0 or err == errno.EISCONN:
                is_connected = True
            if time.time() - start_timeout > self.SOCKET_TIMEOUT:
                time_out = True

        return is_connected

    # ----------------------------------------------------------------------
    def send_command_to_fsbt(self, command):

        try:
            self._fsbt_server.sendall(str(command).encode())
        except Exception as err:
            return

        start_timeout = time.time()
        time_out = False
        got_answer = False
        ans = ''
        while not time_out and not got_answer:
            try:
                ans = str(self._fsbt_server.recv(self.DATA_BUFFER_SIZE).decode())
                got_answer = True
            except socket.error as err:
                if err.errno != 11:
                    time_out = True
                if time.time() - start_timeout > self.SOCKET_TIMEOUT:
                    time_out = True
        if not time_out:
            try:
                ans = json.load(StringIO(ans))
            except Exception as err:
                ans = None
            return ans
        else:
            raise RuntimeError("The FSBT server does not respond")

    # ----------------------------------------------------------------------
    def int_to_bin(self, val):
        b = '{0:04b}'.format(val)
        l = [0] * 4

        for i in range(4):
            l[i] = int(b[i], 2)

        return l


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/ROI.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Roi(object):
    def setupUi(self, Roi):
        Roi.setObjectName("Roi")
        Roi.resize(416, 334)
        self.verticalLayout = QtWidgets.QVBoxLayout(Roi)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(Roi)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.sb_roi_y = QtWidgets.QSpinBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_roi_y.sizePolicy().hasHeightForWidth())
        self.sb_roi_y.setSizePolicy(sizePolicy)
        self.sb_roi_y.setMaximum(9999)
        self.sb_roi_y.setObjectName("sb_roi_y")
        self.gridLayout_3.addWidget(self.sb_roi_y, 2, 3, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBox_3)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 2, 0, 1, 1)
        self.chb_roi_enable = QtWidgets.QCheckBox(self.groupBox_3)
        self.chb_roi_enable.setObjectName("chb_roi_enable")
        self.gridLayout_3.addWidget(self.chb_roi_enable, 0, 0, 1, 1)
        self.sb_roi_x = QtWidgets.QSpinBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_roi_x.sizePolicy().hasHeightForWidth())
        self.sb_roi_x.setSizePolicy(sizePolicy)
        self.sb_roi_x.setMaximum(9999)
        self.sb_roi_x.setObjectName("sb_roi_x")
        self.gridLayout_3.addWidget(self.sb_roi_x, 2, 1, 1, 1)
        self.sb_roi_h = QtWidgets.QSpinBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_roi_h.sizePolicy().hasHeightForWidth())
        self.sb_roi_h.setSizePolicy(sizePolicy)
        self.sb_roi_h.setMaximum(9999)
        self.sb_roi_h.setObjectName("sb_roi_h")
        self.gridLayout_3.addWidget(self.sb_roi_h, 3, 3, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.groupBox_3)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_3.addWidget(self.label_8, 3, 2, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.groupBox_3)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout_3.addWidget(self.label_9, 3, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox_3)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout_3.addWidget(self.label_7, 2, 2, 1, 1)
        self.sb_roi_w = QtWidgets.QSpinBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_roi_w.sizePolicy().hasHeightForWidth())
        self.sb_roi_w.setSizePolicy(sizePolicy)
        self.sb_roi_w.setMaximum(9999)
        self.sb_roi_w.setObjectName("sb_roi_w")
        self.gridLayout_3.addWidget(self.sb_roi_w, 3, 1, 1, 1)
        self.but_delete = QtWidgets.QPushButton(self.groupBox_3)
        self.but_delete.setObjectName("but_delete")
        self.gridLayout_3.addWidget(self.but_delete, 0, 3, 1, 1)
        self.but_color = QtWidgets.QPushButton(self.groupBox_3)
        self.but_color.setMinimumSize(QtCore.QSize(23, 23))
        self.but_color.setMaximumSize(QtCore.QSize(23, 23))
        self.but_color.setText("")
        self.but_color.setObjectName("but_color")
        self.gridLayout_3.addWidget(self.but_color, 0, 1, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_3)
        self.groupBox_5 = QtWidgets.QGroupBox(Roi)
        self.groupBox_5.setCheckable(False)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.le_min_y = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_min_y.sizePolicy().hasHeightForWidth())
        self.le_min_y.setSizePolicy(sizePolicy)
        self.le_min_y.setMinimumSize(QtCore.QSize(0, 0))
        self.le_min_y.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_min_y.setAcceptDrops(False)
        self.le_min_y.setAlignment(QtCore.Qt.AlignCenter)
        self.le_min_y.setObjectName("le_min_y")
        self.gridLayout_4.addWidget(self.le_min_y, 2, 2, 1, 1)
        self.chk_show_max = QtWidgets.QCheckBox(self.groupBox_5)
        self.chk_show_max.setText("")
        self.chk_show_max.setObjectName("chk_show_max")
        self.gridLayout_4.addWidget(self.chk_show_max, 1, 4, 1, 1)
        self.le_min_v = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_min_v.sizePolicy().hasHeightForWidth())
        self.le_min_v.setSizePolicy(sizePolicy)
        self.le_min_v.setMinimumSize(QtCore.QSize(0, 0))
        self.le_min_v.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_min_v.setAcceptDrops(False)
        self.le_min_v.setAlignment(QtCore.Qt.AlignCenter)
        self.le_min_v.setObjectName("le_min_v")
        self.gridLayout_4.addWidget(self.le_min_v, 2, 3, 1, 1)
        self.le_max_v = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_max_v.sizePolicy().hasHeightForWidth())
        self.le_max_v.setSizePolicy(sizePolicy)
        self.le_max_v.setMinimumSize(QtCore.QSize(0, 0))
        self.le_max_v.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_max_v.setAcceptDrops(False)
        self.le_max_v.setAlignment(QtCore.Qt.AlignCenter)
        self.le_max_v.setObjectName("le_max_v")
        self.gridLayout_4.addWidget(self.le_max_v, 1, 3, 1, 1)
        self.le_min_x = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_min_x.sizePolicy().hasHeightForWidth())
        self.le_min_x.setSizePolicy(sizePolicy)
        self.le_min_x.setMinimumSize(QtCore.QSize(0, 0))
        self.le_min_x.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_min_x.setAcceptDrops(False)
        self.le_min_x.setAlignment(QtCore.Qt.AlignCenter)
        self.le_min_x.setObjectName("le_min_x")
        self.gridLayout_4.addWidget(self.le_min_x, 2, 1, 1, 1)
        self.chk_show_min = QtWidgets.QCheckBox(self.groupBox_5)
        self.chk_show_min.setText("")
        self.chk_show_min.setObjectName("chk_show_min")
        self.gridLayout_4.addWidget(self.chk_show_min, 2, 4, 1, 1)
        self.le_com_x = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_com_x.sizePolicy().hasHeightForWidth())
        self.le_com_x.setSizePolicy(sizePolicy)
        self.le_com_x.setMinimumSize(QtCore.QSize(0, 0))
        self.le_com_x.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_com_x.setAcceptDrops(False)
        self.le_com_x.setAlignment(QtCore.Qt.AlignCenter)
        self.le_com_x.setObjectName("le_com_x")
        self.gridLayout_4.addWidget(self.le_com_x, 3, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.groupBox_5)
        self.label_15.setObjectName("label_15")
        self.gridLayout_4.addWidget(self.label_15, 3, 0, 1, 1)
        self.le_com_y = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_com_y.sizePolicy().hasHeightForWidth())
        self.le_com_y.setSizePolicy(sizePolicy)
        self.le_com_y.setMinimumSize(QtCore.QSize(0, 0))
        self.le_com_y.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_com_y.setAcceptDrops(False)
        self.le_com_y.setAlignment(QtCore.Qt.AlignCenter)
        self.le_com_y.setObjectName("le_com_y")
        self.gridLayout_4.addWidget(self.le_com_y, 3, 2, 1, 1)
        self.le_com_v = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_com_v.sizePolicy().hasHeightForWidth())
        self.le_com_v.setSizePolicy(sizePolicy)
        self.le_com_v.setMinimumSize(QtCore.QSize(0, 0))
        self.le_com_v.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_com_v.setAcceptDrops(False)
        self.le_com_v.setAlignment(QtCore.Qt.AlignCenter)
        self.le_com_v.setObjectName("le_com_v")
        self.gridLayout_4.addWidget(self.le_com_v, 3, 3, 1, 1)
        self.chk_show_com = QtWidgets.QCheckBox(self.groupBox_5)
        self.chk_show_com.setText("")
        self.chk_show_com.setObjectName("chk_show_com")
        self.gridLayout_4.addWidget(self.chk_show_com, 3, 4, 1, 1)
        self.le_fwhm_x = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_fwhm_x.sizePolicy().hasHeightForWidth())
        self.le_fwhm_x.setSizePolicy(sizePolicy)
        self.le_fwhm_x.setMinimumSize(QtCore.QSize(0, 0))
        self.le_fwhm_x.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_fwhm_x.setAcceptDrops(False)
        self.le_fwhm_x.setAlignment(QtCore.Qt.AlignCenter)
        self.le_fwhm_x.setObjectName("le_fwhm_x")
        self.gridLayout_4.addWidget(self.le_fwhm_x, 4, 1, 1, 1)
        self.le_sum = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_sum.sizePolicy().hasHeightForWidth())
        self.le_sum.setSizePolicy(sizePolicy)
        self.le_sum.setMinimumSize(QtCore.QSize(0, 0))
        self.le_sum.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_sum.setAcceptDrops(False)
        self.le_sum.setAlignment(QtCore.Qt.AlignCenter)
        self.le_sum.setObjectName("le_sum")
        self.gridLayout_4.addWidget(self.le_sum, 6, 1, 1, 1)
        self.sb_roi_bg = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.sb_roi_bg.setMaximumSize(QtCore.QSize(110, 16777215))
        self.sb_roi_bg.setMaximum(99999999.0)
        self.sb_roi_bg.setObjectName("sb_roi_bg")
        self.gridLayout_4.addWidget(self.sb_roi_bg, 8, 1, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.groupBox_5)
        self.label_16.setObjectName("label_16")
        self.gridLayout_4.addWidget(self.label_16, 4, 0, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.groupBox_5)
        self.label_17.setObjectName("label_17")
        self.gridLayout_4.addWidget(self.label_17, 6, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.groupBox_5)
        self.label_18.setObjectName("label_18")
        self.gridLayout_4.addWidget(self.label_18, 8, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.groupBox_5)
        self.label_14.setObjectName("label_14")
        self.gridLayout_4.addWidget(self.label_14, 2, 0, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.groupBox_5)
        self.label_13.setObjectName("label_13")
        self.gridLayout_4.addWidget(self.label_13, 1, 0, 1, 1)
        self.le_max_x = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_max_x.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_max_x.sizePolicy().hasHeightForWidth())
        self.le_max_x.setSizePolicy(sizePolicy)
        self.le_max_x.setMinimumSize(QtCore.QSize(0, 0))
        self.le_max_x.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_max_x.setAcceptDrops(False)
        self.le_max_x.setAlignment(QtCore.Qt.AlignCenter)
        self.le_max_x.setObjectName("le_max_x")
        self.gridLayout_4.addWidget(self.le_max_x, 1, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.groupBox_5)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.gridLayout_4.addWidget(self.label_10, 0, 1, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.groupBox_5)
        self.label_11.setAlignment(QtCore.Qt.AlignCenter)
        self.label_11.setObjectName("label_11")
        self.gridLayout_4.addWidget(self.label_11, 0, 2, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.groupBox_5)
        self.label_12.setAlignment(QtCore.Qt.AlignCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 0, 3, 1, 1)
        self.le_max_y = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_max_y.sizePolicy().hasHeightForWidth())
        self.le_max_y.setSizePolicy(sizePolicy)
        self.le_max_y.setMinimumSize(QtCore.QSize(0, 0))
        self.le_max_y.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_max_y.setAcceptDrops(False)
        self.le_max_y.setAlignment(QtCore.Qt.AlignCenter)
        self.le_max_y.setObjectName("le_max_y")
        self.gridLayout_4.addWidget(self.le_max_y, 1, 2, 1, 1)
        self.le_fwhm_y = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.le_fwhm_y.sizePolicy().hasHeightForWidth())
        self.le_fwhm_y.setSizePolicy(sizePolicy)
        self.le_fwhm_y.setMinimumSize(QtCore.QSize(0, 0))
        self.le_fwhm_y.setMaximumSize(QtCore.QSize(100, 16777215))
        self.le_fwhm_y.setAcceptDrops(False)
        self.le_fwhm_y.setAlignment(QtCore.Qt.AlignCenter)
        self.le_fwhm_y.setObjectName("le_fwhm_y")
        self.gridLayout_4.addWidget(self.le_fwhm_y, 4, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 4, 3, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.groupBox_5)
        self.label_21.setObjectName("label_21")
        self.gridLayout_4.addWidget(self.label_21, 0, 4, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_5)

        self.retranslateUi(Roi)
        QtCore.QMetaObject.connectSlotsByName(Roi)

    def retranslateUi(self, Roi):
        _translate = QtCore.QCoreApplication.translate
        Roi.setWindowTitle(_translate("Roi", "Form"))
        self.groupBox_3.setTitle(_translate("Roi", "ROI"))
        self.label_6.setText(_translate("Roi", "X:"))
        self.chb_roi_enable.setText(_translate("Roi", "Enable"))
        self.label_8.setText(_translate("Roi", "H:"))
        self.label_9.setText(_translate("Roi", "W:"))
        self.label_7.setText(_translate("Roi", "Y:"))
        self.but_delete.setText(_translate("Roi", "Delete ROI"))
        self.but_color.setToolTip(_translate("Roi", "color"))
        self.groupBox_5.setTitle(_translate("Roi", "Statistics"))
        self.label_15.setText(_translate("Roi", "CoM"))
        self.label_16.setText(_translate("Roi", "FWHM"))
        self.label_17.setText(_translate("Roi", "Sum"))
        self.label_18.setText(_translate("Roi", "BG"))
        self.label_14.setText(_translate("Roi", "Min"))
        self.label_13.setText(_translate("Roi", "Max"))
        self.label_10.setText(_translate("Roi", "X"))
        self.label_11.setText(_translate("Roi", "Y"))
        self.label_12.setText(_translate("Roi", "Value"))
        self.label_21.setText(_translate("Roi", "Mark"))




from subprocess import check_output, STDOUT


def get_version():
    cmd = 'git log -1 --pretty=format:"%ad" --date=format:"%Y-%m-%d %H:%M:%S"'
    try:
        version = check_output(cmd, stderr=STDOUT, shell=True, universal_newlines=True)
        version = version.replace("\n", "")
    except:
        version = 'unknown'
    return version


__version__ = get_version()



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""
This widget displays frame, markers, rois and provide functionality to move markers, rois, as well as center search
"""

import time
import json
import logging

import pyqtgraph as pg
import numpy as np

from datetime import datetime

from PyQt5 import QtCore, QtWidgets, QtGui, QtPrintSupport

from petra_camera.widgets.base_widget import BaseWidget
from petra_camera.gui.FrameViewer_ui import Ui_FrameViewer
from petra_camera.utils.gui_elements import ImageMarker, PeakMarker, LineSegmentItem
from petra_camera.main_window import APP_NAME


WIDGET_NAME = 'FrameViewer'
SAVE_STATE_UIS = ['splitter_y1', 'splitter_y2', 'splitter_x']

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class FrameViewer(BaseWidget):
    """
    """
    new_fps = QtCore.pyqtSignal(float)       # signal, that reports actual FPS to status bar
    cursor_moved = QtCore.pyqtSignal(float, float)  # sends cursor coordinates to status bar

    DEFAULT_IMAGE_EXT = "png"
    FILE_STAMP = "%Y%m%d_%H%M%S"
    DATETIME = "%Y-%m-%d %H:%M:%S"

    LABEL_BRUSH = (30, 144, 255, 170)
    LABEL_COLOR = (255, 255, 255)

    # ----------------------------------------------------------------------
    def __init__(self, parent):
        """
        """
        super(FrameViewer, self).__init__(parent)

        # ----------------------------------------------------------------------
        #                Variables
        # ----------------------------------------------------------------------

        self._last_frame = None # stores last read frame

        self._acq_started = time.time()
        self._fps_counter = 0
        self._set_new_image = True

        self._view_rect = None

        self._marker_widgets = []
        self._rois_widgets = []

        self._hist = None

        # ----------------------------------------------------------------------
        #                UI setup
        # ----------------------------------------------------------------------

        self._ui = Ui_FrameViewer()
        self._ui.setupUi(self)

        self._ui.image_view.ui.histogram.hide()
        self._ui.image_view.ui.roiBtn.hide()
        self._ui.image_view.ui.menuBtn.hide()
        self._ui.wiProfileY.as_projection_y()

        self.add_remove_roi()
        self.repaint_roi()

        self.add_remove_marker()
        self.repaint_marker()

        # ----------------------------------------------------------------------
        #                Settings
        # ----------------------------------------------------------------------

        self._save_data_folder = self._settings.option("save_folder", "default")
        self._save_image_folder = self._settings.option("save_folder", "default")

        # ----------------------------------------------------------------------
        #                Center search functionality
        # ----------------------------------------------------------------------

        # context menu to perform center search
        self._action_first_point = QtWidgets.QAction('Center search start', self)
        self._action_second_point = QtWidgets.QAction('Set second point', self)
        self._action_second_point.setVisible(False)
        self._action_clear_points = QtWidgets.QAction('Clear points', self)
        self._action_clear_points.setEnabled(False)

        self._context_menu = QtWidgets.QMenu()
        self._context_menu.addAction(self._action_first_point)
        self._context_menu.addAction(self._action_second_point)
        self._context_menu.addAction(self._action_clear_points)

        self._center_search_points = [None, None]
        self._search_in_progress = False

        # center search UI
        self._center_search_item = LineSegmentItem('center', float(self._settings.option("center_search", "cross")),
                                                   float(self._settings.option("center_search", "circle")))
        self._center_search_item.setVisible(False)
        self._ui.image_view.view.addItem(self._center_search_item, ignoreBounds=True)

        # ----------------------------------------------------------------------
        #               Peak search functionality
        # ----------------------------------------------------------------------

        self._peak_markers = PeakMarker()
        self._ui.image_view.view.addItem(self._peak_markers, ignoreBounds=True)

        # ----------------------------------------------------------------------
        #                        Ui signals
        # ----------------------------------------------------------------------
        self._ui.image_view.scene.sigMouseMoved.connect(self._mouse_moved)
        self._ui.image_view.scene.sigMouseClicked.connect(self._mouse_clicked)

        self._ui.image_view.view.sigRangeChanged.connect(self._visible_range_changed)
        self._ui.image_view.view.setMenuEnabled(False)

        self._ui.wiProfileX.cursor_moved.connect(lambda x, y: self.cursor_moved.emit(x, y))
        self._ui.wiProfileY.cursor_moved.connect(lambda x, y: self.cursor_moved.emit(x, y))

        # ----------------------------------------------------------------------
        #                        Labels
        # ----------------------------------------------------------------------
        self._device_label = self._add_label('', self._settings.node("camera_viewer/title_label"), visible=True)
        self._device_label.setText(self._camera_device.device_id)

        self._datetime_label = self._add_label("Time", self._settings.node("camera_viewer/datetime_label"),
                                               visible=True)

        self._load_label = self._add_label("Load image", self._settings.node("camera_viewer/datetime_label"),
                                           visible=False)

        if not self.load_camera():
            raise RuntimeError('Cannot set FrameViewer')

        logger.info("FrameView initialized successfully")

    # ----------------------------------------------------------------------
    def get_image_view(self):
        """

        :return: image item to be set to LUT item
        """
        return self._ui.image_view.imageItem

    # ----------------------------------------------------------------------
    def set_hist(self, hist):
        """

        :param hist: LUT item
        :return: None
        """
        self._hist = hist

    # ----------------------------------------------------------------------
    #                        Setting and changing camera device
    # ----------------------------------------------------------------------
    def load_camera(self):
        """
        called to set new camera
        :return: True or False - success or not
        """
        self._set_new_image = True

        # update center search with last saved for camera
        center_search = self._camera_device.get_settings('center_search', str)
        if center_search != '':
            coordinates = json.loads(center_search)
            if coordinates[0] is not None and coordinates[1] is not None and \
                coordinates[2] is not None and coordinates[3] is not None:

                self._center_search_points = [QtCore.QPointF(coordinates[0], coordinates[1]),
                                              QtCore.QPointF(coordinates[2], coordinates[3])]
                self._action_second_point.setVisible(False)
                self._action_clear_points.setEnabled(True)
                self._center_search_item.setVisible(True)
                self._display_center_search()

                return True

        # self._action_second_point.setVisible(False)
        self._action_clear_points.setEnabled(False)
        self._center_search_item.setVisible(False)
        self._display_center_search()

        return True

    # ----------------------------------------------------------------------
    def close(self, auto_screen):
        """

        :param auto_screen: bool, general settings screen control
        :return:
        """
        logger.debug("Closing FrameViewer")

        if self._camera_device and self._camera_device.is_running():
            self._camera_device.stop(auto_screen)

        super(FrameViewer, self).close()

    # ----------------------------------------------------------------------
    def _visible_range_changed(self, viewBox):
        """
        slot for picture zoom signal
        :param viewBox:
        :return: None
        """
        self._view_rect = viewBox.viewRect()

        self._peak_markers.new_scale(self._view_rect.width(), self._view_rect.height())
        self._center_search_item.new_scale(self._view_rect.width(), self._view_rect.height())

        if self._last_frame is not None:

            self._redraw_projections()
            self._show_labels()
            self._camera_device.calculate_roi_statistics()

    # ----------------------------------------------------------------------
    #                  Start stop functionality
    # ----------------------------------------------------------------------
    def start_stop_live_mode(self, auto_screen):
        """

        :param auto_screen: bool, general settings screen control
        :return:
        """
        if self._camera_device and not self._camera_device.is_running():
            self._start_live_mode(auto_screen)
        else:
            self._stop_live_mode(auto_screen)

    # ----------------------------------------------------------------------
    def _start_live_mode(self, auto_screen):
        """

        :param auto_screen: bool, general settings screen control
        :return:
        """
        if self._camera_device:
            self._acq_started = time.time()
            self._fps_counter = 0
            self._set_new_image = True
            self._camera_device.start(auto_screen)
        else:
            QtWidgets.QMessageBox.warning(self, "Initialization Error",
                                      "{} not yet initialized".format(self._camera_device.device_id))

    # ----------------------------------------------------------------------
    def _stop_live_mode(self, auto_screen):
        """

        :param auto_screen: bool, general settings screen control
        :return:
        """
        if self._camera_device and self._camera_device.is_running():
            self._camera_device.stop(auto_screen)
            logger.debug("{} stopped".format(self._camera_device.device_id))

    # ----------------------------------------------------------------------
    #                 Image set and refresh
    # ----------------------------------------------------------------------
    def new_frame(self):
        """
        slot for new frame signal from camera
        :return:
        """
        if hasattr(self, "_load_label"):
            self._load_label.setVisible(True)

        try:
            self._last_frame = self._camera_device.get_frame()

            picture_size = self._camera_device.get_picture_clip()
            reduction = self._camera_device.get_reduction()

            # preparing kwargs for image set or update
            set_kwargs = {'pos': (picture_size[0], picture_size[1]),
                          'scale': (reduction, reduction)}

            if self._camera_device.levels['auto_levels']:
                set_kwargs['autoRange'] = True
                update_kwargs = {'autoLevels': True}
            else:
                set_kwargs['levels'] = (self._camera_device.levels['levels'][0], self._camera_device.levels['levels'][1])
                update_kwargs = {'levels': (self._camera_device.levels['levels'][0], self._camera_device.levels['levels'][1])}

            # we have to disconnect histogram
            self._parent.block_hist_signals(True)

            if self._set_new_image or self._camera_device.set_new_image:
                self._ui.image_view.setImage(self._last_frame, **set_kwargs)
                self._set_new_image = False
                self._camera_device.set_new_image = False
                try:
                    self._ui.image_view.autoRange()
                except:
                    pass
            else:
                self._ui.image_view.imageItem.updateImage(self._last_frame, **update_kwargs)
                self._camera_device.image_need_repaint = False

            self._parent.block_hist_signals(False)

            if self._camera_device.levels['auto_levels']:
                self._camera_device.levels['levels'] = self._hist.getLevels()

            self._redraw_projections()

            self._peak_markers.new_scale(self._view_rect.width(), self._view_rect.height())
            self._center_search_item.new_scale(self._view_rect.width(), self._view_rect.height())

            self._show_labels()

            # FPS counter
            self._fps_counter += 1
            if time.time() - self._acq_started > 1:
                self.new_fps.emit(self._fps_counter)
                self._fps_counter = 0
                self._acq_started = time.time()

        except Exception as err:
            print('Error during refresh view: {}'.format(err))

        if hasattr(self, "_load_label"):
            self._load_label.setVisible(False)

    # ----------------------------------------------------------------------
    def _redraw_projections(self):
        """
        """
        epsilon = 10
        if (self._ui.wiProfileX.frameSize().height() < epsilon and
                self._ui.wiProfileY.frameSize().width() < epsilon):
            return

        # take into account current view range
        x, y = self._view_rect.x(), self._view_rect.y()
        w, h = self._view_rect.width(), self._view_rect.height()

        frameW, frameH = self._last_frame.shape[:2]
        x, y = int(max(0, x)), int(max(0, y))
        w, h = int(min(w, frameW)), int(min(h, frameH))

        dataSlice = self._last_frame[x:x + w, y:y + h]
        if len(dataSlice.shape) > 2:
            dataSlice = (dataSlice[...,0] * 65536 + dataSlice[...,1] * 256 + dataSlice[...,2])/16777215

        if self._ui.wiProfileX.frameSize().height() > epsilon:
            self._ui.wiProfileX.range_changed(dataSlice, 1, (x, y, w, h))

        if self._ui.wiProfileY.frameSize().width() > epsilon:
            self._ui.wiProfileY.range_changed(dataSlice, 0, (x, y, w, h))

    # ----------------------------------------------------------------------
    def repaint_peak_search(self):
        """
        called after peak search is done
        :return:
        """
        self._peak_markers.new_peaks(self._camera_device.peak_coordinates)

    # ----------------------------------------------------------------------
    #                  Marker functionality
    # ----------------------------------------------------------------------

    def add_remove_marker(self):
        """
        called when new roi added or lod deleted

        :return:
        """
        for widget in self._marker_widgets:
            widget.delete_me()

        self._marker_widgets = []
        for ind, marker in enumerate(self._camera_device.markers):
            marker = ImageMarker(marker['x'], marker['y'], self._ui.image_view)
            marker.new_coordinates.connect(lambda x, y, id = ind: self._new_marker_coordinates(id, x, y))
            self._marker_widgets.append(marker)

        self.repaint_marker()

    # ----------------------------------------------------------------------
    def _new_marker_coordinates(self, id, x, y):
        self._camera_device.set_marker_value(id, 'x', x)
        self._camera_device.set_marker_value(id, 'y', y)

    # ----------------------------------------------------------------------
    def repaint_marker(self):
        """
        called when user moves marker or changes color

        :return:
        """
        for widget, marker in zip(self._marker_widgets, self._camera_device.markers):
            widget.setPos(marker['x'], marker['y'])
            widget.setVisible(marker['visible'])
            widget.setColor(marker['color'])

    # ----------------------------------------------------------------------
    #                  ROI functionality
    # ----------------------------------------------------------------------

    def add_remove_roi(self):
        """
        called when new roi added or lod deleted

        :return: None
        """
        for widget, marker, label in self._rois_widgets:
            self._ui.image_view.view.removeItem(widget)
            self._ui.image_view.view.removeItem(marker)
            self._ui.image_view.view.removeItem(label)

        self._rois_widgets = []
        for ind, values in enumerate(self._camera_device.rois):
            widget = pg.RectROI([values['x'], values['y']], [values['w'], values['h']], pen=(0, 9))
            widget.sigRegionChanged.connect(lambda rect, id=ind: self._roi_changed(rect, id))
            if values['visible']:
                widget.hide()
            else:
                widget.show()

            marker_item = LineSegmentItem('cross')
            marker_item.setVisible(values['visible'])
            self._ui.image_view.view.addItem(marker_item, ignoreBounds=True)

            self._ui.image_view.view.addItem(widget, ignoreBounds=True)
            self._rois_widgets.append([widget, marker_item,
                                       self._add_label("ROI_{}".format(ind+1),
                                                       self._settings.node("camera_viewer/roi_label"),
                                                       visible=values['visible'])])

        self.repaint_roi()

    # ----------------------------------------------------------------------
    def repaint_roi(self):
        """
        called when new statistics calculated, or ROI parameters changed

        :return:
        """
        for ind, (roi, cross, label) in enumerate(self._rois_widgets):

            roi.blockSignals(True)

            roi_info = self._camera_device.rois[ind]
            roi_data = self._camera_device.rois_data[ind]

            roi.setVisible(roi_info['visible'])
            label.setVisible(roi_info['visible'])

            roi.setPos([roi_info['x'], roi_info['y']])
            roi.setSize([roi_info['w'], roi_info['h']])
            if roi_info['color']:
                roi.setPen(roi_info['color'])

            label.setPos(roi_info['x'] + roi_info['w'],
                         roi_info['y'] + roi_info['h'])

            cross.setVisible(False)
            if roi_info['visible']:
                if roi_info['mark'] == 'max':
                    cross.set_pos((roi_data['max_x'], roi_data['max_y']), (roi_data['fwhm_x'], roi_data['fwhm_y']))
                elif roi_info['mark'] == 'min':
                    cross.set_pos((roi_data['min_x'], roi_data['min_y']), (roi_data['fwhm_x'], roi_data['fwhm_y']))
                elif roi_info['mark'] == 'com':
                    cross.set_pos((roi_data['com_x'], roi_data['com_y']), (roi_data['fwhm_x'], roi_data['fwhm_y']))
                cross.setVisible(True)

            roi.blockSignals(False)

    # ----------------------------------------------------------------------
    def _roi_changed(self, rect, ind):
        """
        called when ROI emits sigRegionChanged signal.
        :param rect: new roi rect
        :param ind: ROIs index
        :return:
        """
        if self._last_frame is not None:
            pos, size = rect.pos(), rect.size()
            max_w, max_h = self._last_frame.shape[:2]

            self._camera_device.set_roi_value(ind, 'x', max(int(pos.x()), 0))
            self._camera_device.set_roi_value(ind, 'y', max(int(pos.y()), 0))
            self._camera_device.set_roi_value(ind, 'w', min(int(size.x()), max_w))
            self._camera_device.set_roi_value(ind, 'h', min(int(size.y()), max_h))

            self._camera_device.calculate_roi_statistics()

            self._rois_widgets[ind][2].setPos(pos.x() + size.x(), pos.y() + size.y())

    # ----------------------------------------------------------------------
    #                  Center search functionality
    # ----------------------------------------------------------------------
    def _mouse_moved(self, pos):
        """
        slot for mouse events, utilized for center search
        :param pos: mouse position
        :return:
        """
        pos = self._ui.image_view.view.mapSceneToView(pos)
        self.cursor_moved.emit(pos.x(), pos.y())
        if self._search_in_progress:
            self._center_search_points[1] = pos
            self._display_center_search()

    # ----------------------------------------------------------------------
    def _mouse_clicked(self, event):
        """
        slot for mouse events, mainly utilized for center search
        :param event:
        :return:
        """
        if event.double():
            try:
                self._ui.image_view.autoRange()
            except:
                pass

        elif event.button() == 2:  # right click

            action = self._context_menu.exec_(event._screenPos)

            if action == self._action_first_point:
                self._center_search_points[0] = self._ui.image_view.view.mapSceneToView(event.scenePos())
                self._search_in_progress = True
                self._action_second_point.setVisible(True)
                self._action_clear_points.setEnabled(True)
                self._center_search_item.setVisible(True)

            elif action == self._action_second_point:
                self._center_search_points[1] = self._ui.image_view.view.mapSceneToView(event.scenePos())
                self._action_second_point.setVisible(False)
                self._search_in_progress = False
                self._save_center_search()

            else:
                self._center_search_points = [None, None]
                self._action_second_point.setVisible(False)
                self._action_clear_points.setEnabled(False)
                self._center_search_item.setVisible(False)
                self._search_in_progress = False
                self._save_center_search()

        elif event.button() == 1 and self._search_in_progress:  # left click

            self._center_search_points[1] = self._ui.image_view.view.mapSceneToView(event.scenePos())
            self._action_second_point.setVisible(False)
            self._search_in_progress = False
            self._save_center_search()

        self._display_center_search()

    # ----------------------------------------------------------------------
    def _save_center_search(self):
        """
        drops center search parameters
        :return: None
        """
        if self._center_search_points[0] is not None and self._center_search_points[1] is not None:
            coordinates = [self._center_search_points[0].x(), self._center_search_points[0].y(),
                           self._center_search_points[1].x(), self._center_search_points[1].y()]
        else:
            coordinates = [None, None]
        self._camera_device.save_settings('center_search', json.dumps(coordinates))

    # ----------------------------------------------------------------------
    def _display_center_search(self):
        """

        :return:
        """
        self._center_search_item.set_pos(self._center_search_points)

    # ----------------------------------------------------------------------
    # ---------------- Image save and print functionality ------------------
    # ----------------------------------------------------------------------

    def _show_labels(self):
        """
        """
        if hasattr(self, "_device_label"):
            self._show_label(0.5, 0.04, self._device_label)

        if hasattr(self, "_datetime_label"):
            msg = datetime.now().strftime(self.DATETIME)
            self._datetime_label.setText(msg)

            self._show_label(0.85, 0.9, self._datetime_label)

        if hasattr(self, "_load_label"):
            self._show_label(0.85, 0.04, self._load_label)

    # ----------------------------------------------------------------------
    def _add_label(self, text, style=None, visible=True):
        """

        :param text: str
        :param style: style sheet
        :param visible: bool
        :return: pg text item
        """
        if not style:
            color = self.LABEL_COLOR
            fill = self.LABEL_BRUSH
            font = QtGui.QFont("Arial", 10)
        else:
            color = tuple(int(v) for v in style.getAttribute("fg_color").split(","))
            fill = tuple(int(v) for v in style.getAttribute("bg_color").split(","))
            font = style.getAttribute("font").split(",")
            font = QtGui.QFont(font[0], int(font[1]))

        item = pg.TextItem(text=text, color=color, fill=fill)
        item.setFont(font)
        item.setVisible(visible)

        self._ui.image_view.view.addItem(item, ignoreBounds=True)

        return item

    # ----------------------------------------------------------------------
    def _show_label(self, x, y, label):
        """
        Args:
            x, y (float), normalized to 0-1 range position
        """
        [[xMin, xMax], [yMin, yMax]] = self._ui.image_view.view.viewRange()

        deltaX = abs(xMax - xMin)
        textX = xMax - deltaX * (1. - x)

        deltaY = abs(yMax - yMin)
        textY = yMax - deltaY * (1. - y)

        label.setPos(textX, textY)

    # ----------------------------------------------------------------------
    # ---------------- Image save and print functionality ------------------
    # ----------------------------------------------------------------------
    def save_to_image(self):
        """
        """
        self.stop_live_mode()

        fileName = self._get_image_file_name("Save Image")
        if fileName:
            pixmap = QtGui.QScreen.grabWidget(self._ui.image_view)
            pixmap.save(fileName)

    # ----------------------------------------------------------------------

    def save_to_file(self, fmt):
        """Saves to text file or numpy's npy/npz.
        """
        self.stop_live_mode()

        fmt = fmt.lower()
        defaultName = "data_{}.{}".format(datetime.now().strftime(self.FILE_STAMP),
                                          fmt)

        fileTuple, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Save To File", self._save_data_folder + defaultName,
                                                             filter=(self.tr("Ascii Files (*.csv)")
                                                              if fmt == "csv" else
                                                              self.tr("Numpy Files (*.npy)")))

        self._save_data_folder = QtCore.QFileInfo(fileTuple).path() + '/'

        fileName = str(fileTuple)
        fileName = fileName.strip()

        if fileName:
            data, scale = self._camera_device.get_frame()  # sync with data acq! TODO

            if fmt.lower() == "csv":
                np.savetxt(fileName, data)
            elif fmt.lower() == "npy":
                np.save(fileName, data)
            else:
                raise ValueError("Unknown format '{}'".format(fmt))

    # ---------------------------------------------------------------------- 
    def print_image(self):
        """
        """
        self.stop_live_mode()

        self._printer = QtPrintSupport.QPrinter()

        if QtPrintSupport.QPrintDialog(self._printer).exec_() == QtWidgets.QDialog.Accepted:
            self._printPainter = QtGui.QPainter(self._printer)
            self._printPainter.setRenderHint(QtGui.QPainter.Antialiasing)

            self._ui.image_view.view.render(self._printPainter)

    # ---------------------------------------------------------------------- 
    def to_clipboard(self):
        """NOTE that the content of the clipboard is cleared after program's exit.
        """
        self.stop_live_mode()

        pixmap = QtGui.QScreen.grabWidget(self._ui.image_view)
        QtWidgets.qApp.clipboard().setPixmap(pixmap)

    # ----------------------------------------------------------------------
    def _get_image_file_name(self, title):
        """
        """
        filesFilter = ";;".join(["(*.{})".format(ffilter) for ffilter in
                                 QtGui.QImageWriter.supportedImageFormats()])

        defaultName = "image_{}.{}".format(datetime.now().strftime(self.FILE_STAMP),
                                           self.DEFAULT_IMAGE_EXT)
        fileTuple, _ = QtWidgets.QFileDialog.getSaveFileName(self, title, self._save_image_folder + defaultName,
                                                             filesFilter)

        self._save_image_folder = QtCore.QFileInfo(fileTuple).path() + '/'

        return str(fileTuple)



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""
"""
APP_NAME = "2DCameraViewer"

import getpass
import shutil
import logging
import os
import psutil
import socket
import pyqtgraph as pg
from pathlib import Path

from PyQt5 import QtWidgets, QtCore

from distutils.util import strtobool

from petra_camera.widgets.about_dialog import AboutDialog
from petra_camera.widgets.camera_widget import CameraWidget
from petra_camera.utils.xmlsettings import XmlSettings

from petra_camera.roisrv.roiserver import RoiServer

from petra_camera.gui.MainWindow_ui import Ui_MainWindow

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class PETRACamera(QtWidgets.QMainWindow):
    """
    """
    LOG_PREVIEW = "gvim"
    STATUS_TICK = 2000              # [ms]

    # ----------------------------------------------------------------------
    def __init__(self, options):
        """
        """
        super(PETRACamera, self).__init__()
        self._ui = Ui_MainWindow()
        self._ui.setupUi(self)

        self._init_menu()
        self._load_ui_settings()

        pg.setConfigOption("background", "w")
        pg.setConfigOption("foreground", "k")
        pg.setConfigOption("leftButtonPan", False)

        self.options = options
        self._settings = self.get_settings(options)
        self._device_list = self._get_cameras_list()
        self._camera_widgets = []
        self._camera_docks = []
        self._add_cameras()

        self._roi_server = []
        if self._settings.has_node('roi_server') and self._settings.option("roi_server", "enable").lower() == "true":
            try:
                self._roi_server = RoiServer(self._settings.option("roi_server", "host"),
                                             self._settings.option("roi_server", "port"),
                                             self._device_list)
                self._roi_server.start()
            except Exception as err:
                logger.exception(err)

        self.setWindowTitle("Camera Viewer ({}@{})".format(getpass.getuser(), socket.gethostname()))

        self._init_status_bar()
        self._status_timer = QtCore.QTimer(self)
        self._status_timer.timeout.connect(self._refresh_status_bar)
        self._status_timer.start(self.STATUS_TICK)

        logger.info("Initialized successfully")

    # ----------------------------------------------------------------------
    def get_settings(self, options):

        home = os.path.join(str(Path.home()), '.petra_camera')
        file_name = str(options.profile)
        if not file_name.endswith('.xml'):
            file_name += '.xml'

        if not os.path.exists(os.path.join(home, file_name)):
            file = QtWidgets.QFileDialog.getOpenFileName(self, 'Cannot find settings file, please locate it',
                                                         str(Path.home()), 'XML settings (*.xml)')
            if file[0]:
                return XmlSettings(file[0])
        else:
            return XmlSettings(os.path.join(home, file_name))

        if not os.path.exists(home):
            os.mkdir(home)

        if not os.path.exists(os.path.join(home, file_name)):
            shutil.copy(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'default_config.xml'),
                        os.path.join(home, 'default.xml'))

        return XmlSettings(os.path.join(home, 'default.xml'))

    # ----------------------------------------------------------------------
    def _get_cameras_list(self):

        cam_list = []
        for device in self._settings.get_nodes('camera_viewer', 'camera'):
            cam_list.append(device.getAttribute('name'))

        cam_list.sort()

        return cam_list

    # ----------------------------------------------------------------------
    def _add_cameras(self):
        """
        Here we making cameras widgets and docking them
        :return:
        """
        self.setCentralWidget(None)

        self.setDockOptions(QtWidgets.QMainWindow.AnimatedDocks |
                            QtWidgets.QMainWindow.AllowNestedDocks |
                            QtWidgets.QMainWindow.AllowTabbedDocks)

        self.setTabPosition(QtCore.Qt.LeftDockWidgetArea, QtWidgets.QTabWidget.North)

        for camera in self._device_list:
            try:
                widget, dock = self.add_dock(CameraWidget, f"{camera}", self, self._settings, camera)
                widget.load_ui_settings()
                dock.setStyleSheet("""QDockWidget {font-size: 14pt; font-weight: bold;}""")
                self._camera_widgets.append(widget)
                self._camera_docks.append(dock)

            except Exception as err:
                open_mgs = QtWidgets.QMessageBox()
                open_mgs.setIcon(QtWidgets.QMessageBox.Critical)
                open_mgs.setWindowTitle(f"Error")
                open_mgs.setText(f"Cannot add {camera}:\n{err}")
                open_mgs.setStandardButtons(QtWidgets.QMessageBox.Ok)
                open_mgs.exec_()

    # ----------------------------------------------------------------------
    def add_dock(self, WidgetClass, label, *args, **kwargs):
        """
        """
        widget = WidgetClass(*args, **kwargs)

        dock = QtWidgets.QDockWidget(label)
        dock.setObjectName("{0}Dock".format("".join(label.split())))
        dock.setWidget(widget)

        children = [child for child in self.findChildren(QtWidgets.QDockWidget)
                    if isinstance(child.widget(), CameraWidget)]
        if children:
            self.tabifyDockWidget(children[-1], dock)
        else:
            self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dock)

        self.menu_cameras.addAction(dock.toggleViewAction())

        return widget, dock

    # ----------------------------------------------------------------------
    def _show_about(self):
        """
        """
        AboutDialog(self).exec_()

    # ----------------------------------------------------------------------
    def closeEvent(self, event):
        """
        """
        if self.clean_close():
            event.accept()
        else:
            event.ignore()

    # ----------------------------------------------------------------------
    def clean_close(self):
        """
        """
        logger.info("Closing the app...")

        for widget in self._camera_widgets:
            widget.clean_close()

        if hasattr(self, '_roi_server') and self._roi_server:
            logger.info("Stopping ROI server...")
            self._roi_server.stop()

        if hasattr(self, '_status_timer'):
            self._status_timer.stop()

        self._save_ui_settings()

        QtWidgets.qApp.clipboard().clear()
        logger.info("Closed properly")

        return True

    # ----------------------------------------------------------------------
    def _quit_program(self):
        """
        """
        if self.clean_close():
            QtWidgets.qApp.quit()
            # pass

    # ----------------------------------------------------------------------
    def _save_ui_settings(self):
        """Save basic GUI settings.
        """
        settings = QtCore.QSettings(APP_NAME)

        settings.setValue("MainWindow/geometry", self.saveGeometry())
        settings.setValue("MainWindow/state", self.saveState())

        settings.setValue("AutoScreen", self.auto_screen_action.isChecked())

    # ----------------------------------------------------------------------
    def _load_ui_settings(self):
        """Load basic GUI settings.
        """
        settings = QtCore.QSettings(APP_NAME)

        try:
            self.restoreGeometry(settings.value("MainWindow/geometry"))
        except:
            pass

        try:
            self.restoreState(settings.value("MainWindow/state"))
        except:
            pass

        enable_auto_screens = False
        try:
            enable_auto_screens = strtobool(settings.value("AutoScreen"))
        except:
            pass

        self.auto_screen_action.setChecked(enable_auto_screens)
        self._display_auto_screens(enable_auto_screens)

    # ----------------------------------------------------------------------
    def _display_auto_screens(self, state):
        font = self.auto_screen_action.font()

        if state:
            self.auto_screen_action.setText('Auto screens ENABLED')
        else:
            self.auto_screen_action.setText('Auto screens DISABLED')

        font.setBold(state)
        self.auto_screen_action.setFont(font)

    # ----------------------------------------------------------------------
    def _init_menu(self):

        self.menu_cameras = QtWidgets.QMenu('Cameras', self)
        self.menuBar().addMenu(self.menu_cameras)

        self.auto_screen_action = QtWidgets.QAction('', self)
        self.auto_screen_action.setCheckable(True)
        self.auto_screen_action.triggered.connect(self._display_auto_screens)
        self.menuBar().addAction(self.auto_screen_action)

        about_action = QtWidgets.QAction('About', self)
        about_action.triggered.connect(self._show_about)
        self.menuBar().addAction(about_action)

        quit_action = QtWidgets.QAction('Exit', self)
        quit_action.triggered.connect(self._quit_program)
        self.menuBar().addAction(quit_action)

    # ----------------------------------------------------------------------
    def _init_status_bar(self):
        """
        """

        processID = os.getpid()
        currentDir = os.getcwd()

        lbProcessID = QtWidgets.QLabel("PID {}".format(processID))
        lbProcessID.setStyleSheet("QLabel {color: #000066;}")
        lbCurrentDir = QtWidgets.QLabel("{}".format(currentDir))

        # resource usage
        process = psutil.Process(processID)
        mem = float(process.memory_info().rss) / (1024. * 1024.)
        cpu = process.cpu_percent()

        self._lb_resources_status = QtWidgets.QLabel("| {:.2f}MB | CPU {} % |".format(mem, cpu))


        self.statusBar().addPermanentWidget(lbProcessID)
        self.statusBar().addPermanentWidget(lbCurrentDir)
        self.statusBar().addPermanentWidget(self._lb_resources_status)

    # ----------------------------------------------------------------------
    def _refresh_status_bar(self):
        """
        """
        process = psutil.Process(os.getpid())
        mem = float(process.memory_info().rss) / (1024. * 1024.)
        cpu = psutil.cpu_percent()

        self._lb_resources_status.setText("| {:.2f}MB | CPU {} % |".format(mem,
                                                                           cpu))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/ProjectionWidget.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ProjectionWidget(object):
    def setupUi(self, ProjectionWidget):
        ProjectionWidget.setObjectName("ProjectionWidget")
        ProjectionWidget.resize(849, 123)
        self.verticalLayout = QtWidgets.QVBoxLayout(ProjectionWidget)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.graphicsView = GraphicsView(ProjectionWidget)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout.addWidget(self.graphicsView)

        self.retranslateUi(ProjectionWidget)
        QtCore.QMetaObject.connectSlotsByName(ProjectionWidget)

    def retranslateUi(self, ProjectionWidget):
        _translate = QtCore.QCoreApplication.translate
        ProjectionWidget.setWindowTitle(_translate("ProjectionWidget", "Form"))

from pyqtgraph import GraphicsView



#!/usr/bin/python3
import io
import os
from subprocess import check_output, STDOUT
from setuptools.command.install import install

from setuptools import setup, find_packages


class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        print("Run pre-setup")
        # Set version
        cmd = 'git log -1 --pretty=format:"%ad" --date=format:"%Y-%m-%d %H:%M:%S"'
        try:
            version = check_output(cmd, stderr=STDOUT, shell=True, universal_newlines=True)
            version = version.replace("\n", "")
        except:
            version = 'unknown'
        file = open('petra_camera/version.py', 'w')
        file.write('__version__="{}"'.format(version))
        file.close()

        # build qt gui
        # cmd = "python3 .petra_viewer/compile_uis.py"
        # output = check_output(cmd, stderr=STDOUT, shell=True, universal_newlines=True)
        # print(output)
        install.run(self)


# Package meta-data.
NAME = 'petra_camera'
DESCRIPTION = 'Simple viewer for cameras, used at PETRA III source'
EMAIL = 'yury.matveev@desy.de'
AUTHOR = 'Yury Matveyev'
REQUIRES_PYTHON = '>=3.7'
VERSION = '0.4.0'

# What packages are required for this module to be executed?
REQUIRED = ['pyqtgraph', 'psutil', 'numpy', 'scipy',
]

EXTRA_REQUIRED = {'LAMBDA': ['watchdog'],
                  'PEAK': ['scikit-image']
                  }

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
here = os.path.abspath(os.path.dirname(__file__))
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except IOError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
about['__version__'] = VERSION

# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    packages=find_packages(),
    package_dir={'petra_camera': 'petra_camera',
                 },
    package_data={
        'petra_camera': ['petra_camera/*.py', 'petra_camera/*.xml' ],
    },
    cmdclass={
        'install': PostInstallCommand,
    },
    install_requires=REQUIRED,
    extras_require=EXTRA_REQUIRED,
    include_package_data=True,
    license='GPLv3',
    entry_points={
        'console_scripts': [
            'petra_camera = petra_camera:main',
        ],
    },

    scripts=['petra_camera/petra_camera.sh'],

    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Development Status :: 3 - Alpha'
    ],
)



# Created by matveyev at 01.12.2020

# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------


"""Dalsa camera proxy
"""

import numpy as np
import PyTango
import logging
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

import os.path as ospath

from PIL import Image

from petra_camera.devices.base_camera import BaseCamera
from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)

# ----------------------------------------------------------------------
class DalsaProxy(BaseCamera):

    _settings_map = {
                     'max_level_limit': (None, )
                     }

    visible_layouts = ('folder', 'source')

    # ----------------------------------------------------------------------
    def __init__(self, settings):
        super(DalsaProxy, self).__init__(settings)

        if settings.hasAttribute('folders'):
            self._possible_folders = [item.strip() for item in settings.getAttribute("folders").split(';')]
        else:
            self._possible_folders = ['/gpfs/current/raw/', '/gpfs/commissioning/raw/']

        if settings.hasAttribute('sources'):
            self._possible_sources = [item.strip() for item in settings.getAttribute("folders").split(';')]
        else:
            self._possible_sources = ['Event', 'Files']

        self._my_event_handler = PatternMatchingEventHandler(["*.tif"], "", False, True)
        self._my_event_handler.on_created = self._on_created

        self._my_observer = None

        self._source = self._possible_sources[0]

        self.path = self._possible_folders[0]

        self._last_frame = np.zeros((1, 1))

        self.error_flag = False
        self.error_msg = ''

        self._running = False

    # ----------------------------------------------------------------------
    def start_acquisition(self):

        if self._source == 'Event':

            logger.debug(f'{self._my_name}: starting acquisition: event mode')

            if self._device_proxy is None:
                raise RuntimeError('No device proxy')

            self._device_proxy.write_attribute("PixelFormat", "Mono16")
            self._device_proxy.write_attribute("ViewingMode", 2)
            self._eid = self._device_proxy.subscribe_event("Image16", PyTango.EventType.DATA_READY_EVENT,
                                                           self._on_event, [], True)
            self._running = True
            return True

        elif self._source == 'Files':
            if self.path != '':

                logger.debug(f'{self._my_name}: starting acquisition: file mode')

                self._my_observer = Observer()
                self._my_observer.schedule(self._my_event_handler, self.path, recursive=True)
                self._my_observer.start()
                self._running = True
                return True
            else:
                raise RuntimeError('Path is not exist')
        else:
            raise RuntimeError('Unknown mode')

    # ----------------------------------------------------------------------
    def stop_acquisition(self):

        if self._source == 'Event':
            logger.debug(f'{self._my_name}: stopping acquisition: event mode')
            self._device_proxy.unsubscribe_event(self._eid)

        elif self._source == 'Files':

            logger.debug(f'{self._my_name}: stopping acquisition: file mode')
            self._my_observer.stop()
            self._my_observer.join()

        else:
            raise RuntimeError('Unknown mode')

        self._running = False

    # ----------------------------------------------------------------------
    def _on_event(self, event):

        if not event.err:

            logger.debug(f'{self._my_name}: new tango event')

            data = event.device.read_attribute(event.attr_name.split('/')[6])
            self._last_frame = np.array(data.value)[self._picture_size[0]:self._picture_size[2],
                                                    self._picture_size[1]:self._picture_size[3]]

            self._new_frame_flag = True
        else:
            pass

    # ----------------------------------------------------------------------
    def _on_created(self, event):

        logger.debug(f'{self._my_name}: new file system event')

        self.id = ' file: {}'.format(ospath.splitext(ospath.basename(event.src_path))[0])

        self._last_frame = np.array(Image.open(event.src_path))[self._picture_size[0]:self._picture_size[2],
                                                                self._picture_size[1]:self._picture_size[3]]
        self._new_frame_flag = True

    # ----------------------------------------------------------------------
    def _set_new_path(self, path):

        logger.debug(f'{self._my_name}: new file path: {path}')

        need_to_restart = self._running
        if self._running:
            self.stop_acquisition()
            self._last_frame = np.zeros((1, 1))
            self._new_frame_flag = True

        self.path = path

        if need_to_restart:
            self.start_acquisition()

    # ----------------------------------------------------------------------
    def get_settings(self, option, cast):

        if option in ['Path', 'Source', 'possible_sources', 'possible_folders']:

            logger.debug(f'{self._my_name}: setting {cast.__name__}({option}) requested')

            if option == 'Path':
                path = super(DalsaProxy, self).get_settings(option, cast)
                if path != '':
                    self._set_new_path(path)
                return self.path

            elif option == 'Source':
                source = super(DalsaProxy, self).get_settings(option, cast)
                if source != '':
                    self._change_source(source)
                return self._source

            elif option == 'possible_sources':

                return self._possible_sources

            elif option == 'possible_folders':
                return self._possible_folders

        else:
            return super(DalsaProxy, self).get_settings(option, cast)

    # ----------------------------------------------------------------------
    def save_settings(self, option, value):

        if option in ['Path', 'Source']:
            logger.debug(f'{self._my_name}: setting {option}: new value {value}')

            if option == 'Path':
                self._set_new_path(value)

            elif option == 'Source':
                self._change_source(value)

        super(DalsaProxy, self).save_settings(option, value)

    # ----------------------------------------------------------------------
    def _change_source(self, source):
        need_to_restart = self._running
        if self._running:
            self.stop_acquisition()
            self._last_frame = np.zeros((1, 1))
            self._new_frame_flag = True

        self._source = source

        if need_to_restart:
            self.start_acquisition()


# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------


"""TangotoTine camera proxy
"""

import numpy as np
import time
import logging
from threading import Thread
try:
    import PyTango
except ImportError:
    pass

from petra_camera.devices.base_camera import BaseCamera
from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class TangoTineProxy(BaseCamera):
    """Proxy to a physical TANGO device.
    """
    # SERVER_SETTINGS = {"PixelFormat": "Mono8",  # possibly more...
    #                    "ViewingMode": 1}

    _settings_map = {
                     "exposure": ("device_proxy", "ExposureTime"),
                     "gain": ("device_proxy", "Gain"),
                     'max_level_limit': (None, ),
                     "counter_x": ('roi_server', 'roi_x'),
                     "counter_y": ('roi_server', 'roi_y'),
                     "counter_w": ('roi_server', 'roi_w'),
                     "counter_h": ('roi_server', 'roi_h'),
                     "background": ("device_proxy", "SubtractBackground"),
                     "background_sigmas": ("device_proxy", "BackgroundSigmas"),
                     }

    visible_layouts = ('FPS', 'exposure', 'background')

    # ----------------------------------------------------------------------
    def __init__(self, settings):
        super(TangoTineProxy, self).__init__(settings)

        self._last_frame = np.zeros((1, 1))

        self.error_flag = False
        self.error_msg = ''

        self.period = 1/self.get_settings('FPS', int)

        self._camera_read_thread = None
        self._camera_read_thread_running = False

    # ----------------------------------------------------------------------
    def start_acquisition(self):
        """
        start acquisition tread
        :return: bool
        """

        logger.debug(f"{self._my_name} starting thread")

        try:
            self._camera_read_thread = Thread(target=self._readout_frame)
            self._camera_read_thread_running = True
            self._camera_read_thread.start()

            return True

        except Exception as err:

            logger.exception(err)

        return False

    # ----------------------------------------------------------------------
    def stop_acquisition(self):
        """
        stops acquisition tread
        :return:
        """

        if self._camera_read_thread_running:
            self._camera_read_thread_running = False
            self._camera_read_thread.join()

        logger.debug(f"{self._my_name} thread stopped")

    # ----------------------------------------------------------------------
    def is_running(self):
        """

        :return: bool
        """
        return self._camera_read_thread_running

    # ----------------------------------------------------------------------
    def _readout_frame(self):
        """Called each time new frame is available.
        """

        while self._camera_read_thread_running:
            try:
                self._last_frame = self._device_proxy.Frame[self._picture_size[0]:self._picture_size[2],
                                                            self._picture_size[1]:self._picture_size[3]]
                self._new_frame_flag = True
                logger.debug(f"{self._my_name} new frame")

                time.sleep(self.period)

            except Exception as err:

                logger.exception(f"{self._my_name} exception during new frame: {err}")
                self._camera_read_thread_running = False

    # ----------------------------------------------------------------------
    def get_settings(self, option, cast):
        """
        here we catch some settings before read them from general settings
        :param option:
        :param cast:
        :return:
        """

        if option in ['max_width', 'max_height', 'FPSmax']:

            logger.debug(f'{self._my_name}: setting {cast.__name__}({option}) requested')

            if option in ['max_width', 'max_height']:

                w, h = self._device_proxy.Frame.shape
                if option == 'max_width':
                    return w
                else:
                    return h

            elif option == 'FPSmax':
                return 1000/self.period

        elif option == 'FPS':
            return max(1, super(TangoTineProxy, self).get_settings(option, cast))

        else:
            return super(TangoTineProxy, self).get_settings(option, cast)